@@ -36,10 +36,16 @@ def _all_related_fields(model):
 def _related_non_m2m_objects(old_field, new_field):
     # Filter out m2m objects from reverse relations.
     # Return (old_relation, new_relation) tuples.
-    return zip(
+    related_fields = zip(
         (obj for obj in _all_related_fields(old_field.model) if _is_relevant_relation(obj, old_field)),
         (obj for obj in _all_related_fields(new_field.model) if _is_relevant_relation(obj, new_field)),
     )
+    for old_rel, new_rel in related_fields:
+        yield old_rel, new_rel
+        yield from _related_non_m2m_objects(
+            old_rel.remote_field,
+            new_rel.remote_field,
+        )
 
 
 class BaseDatabaseSchemaEditor:
@@ -135,7 +141,7 @@ class BaseDatabaseSchemaEditor:
         # Log the command we're running, then run it
         logger.debug("%s; (params %r)", sql, params, extra={'params': params, 'sql': sql})
         if self.collect_sql:
-            ending = "" if sql.endswith(";") else ";"
+            ending = "" if sql.rstrip().endswith(";") else ";"
             if params is not None:
                 self.collected_sql.append((sql % tuple(map(self.quote_value, params))) + ending)
             else:
@@ -150,10 +156,10 @@ class BaseDatabaseSchemaEditor:
     def table_sql(self, model):
         """Take a model and return its table definition."""
         # Add any unique_togethers (always deferred, as some fields might be
-        # created afterwards, like geometry fields with some backends).
-        for fields in model._meta.unique_together:
-            columns = [model._meta.get_field(field).column for field in fields]
-            self.deferred_sql.append(self._create_unique_sql(model, columns))
+        # created afterward, like geometry fields with some backends).
+        for field_names in model._meta.unique_together:
+            fields = [model._meta.get_field(field) for field in field_names]
+            self.deferred_sql.append(self._create_unique_sql(model, fields))
         # Create column SQL, add FK deferreds if needed.
         column_sqls = []
         params = []
@@ -206,58 +212,68 @@ class BaseDatabaseSchemaEditor:
 
     # Field <-> database mapping functions
 
-    def column_sql(self, model, field, include_default=False):
-        """
-        Take a field and return its column definition.
-        The field must already have had set_attributes_from_name() called.
-        """
-        # Get the column's type and use that as the basis of the SQL
-        db_params = field.db_parameters(connection=self.connection)
-        sql = db_params['type']
-        params = []
-        # Check for fields that aren't actually columns (e.g. M2M)
-        if sql is None:
-            return None, None
-        # Collation.
+    def _iter_column_sql(self, column_db_type, params, model, field, include_default):
+        yield column_db_type
         collation = getattr(field, 'db_collation', None)
         if collation:
-            sql += self._collate_sql(collation)
-        # Work out nullability
+            yield self._collate_sql(collation)
+        # Work out nullability.
         null = field.null
-        # If we were told to include a default value, do so
-        include_default = include_default and not self.skip_default(field)
+        # Include a default value, if requested.
+        include_default = (
+            include_default and
+            not self.skip_default(field) and
+            # Don't include a default value if it's a nullable field and the
+            # default cannot be dropped in the ALTER COLUMN statement (e.g.
+            # MySQL longtext and longblob).
+            not (null and self.skip_default_on_alter(field))
+        )
         if include_default:
             default_value = self.effective_default(field)
-            column_default = ' DEFAULT ' + self._column_default_sql(field)
             if default_value is not None:
+                column_default = 'DEFAULT ' + self._column_default_sql(field)
                 if self.connection.features.requires_literal_defaults:
-                    # Some databases can't take defaults as a parameter (oracle)
+                    # Some databases can't take defaults as a parameter (Oracle).
                     # If this is the case, the individual schema backend should
-                    # implement prepare_default
-                    sql += column_default % self.prepare_default(default_value)
+                    # implement prepare_default().
+                    yield column_default % self.prepare_default(default_value)
                 else:
-                    sql += column_default
-                    params += [default_value]
+                    yield column_default
+                    params.append(default_value)
         # Oracle treats the empty string ('') as null, so coerce the null
         # option whenever '' is a possible value.
         if (field.empty_strings_allowed and not field.primary_key and
                 self.connection.features.interprets_empty_strings_as_nulls):
             null = True
-        if null and not self.connection.features.implied_column_null:
-            sql += " NULL"
-        elif not null:
-            sql += " NOT NULL"
-        # Primary key/unique outputs
+        if not null:
+            yield 'NOT NULL'
+        elif not self.connection.features.implied_column_null:
+            yield 'NULL'
         if field.primary_key:
-            sql += " PRIMARY KEY"
+            yield 'PRIMARY KEY'
         elif field.unique:
-            sql += " UNIQUE"
-        # Optionally add the tablespace if it's an implicitly indexed column
+            yield 'UNIQUE'
+        # Optionally add the tablespace if it's an implicitly indexed column.
         tablespace = field.db_tablespace or model._meta.db_tablespace
         if tablespace and self.connection.features.supports_tablespaces and field.unique:
-            sql += " %s" % self.connection.ops.tablespace_sql(tablespace, inline=True)
-        # Return the sql
-        return sql, params
+            yield self.connection.ops.tablespace_sql(tablespace, inline=True)
+
+    def column_sql(self, model, field, include_default=False):
+        """
+        Return the column definition for a field. The field must already have
+        had set_attributes_from_name() called.
+        """
+        # Get the column's type and use that as the basis of the SQL.
+        db_params = field.db_parameters(connection=self.connection)
+        column_db_type = db_params['type']
+        # Check for fields that aren't actually columns (e.g. M2M).
+        if column_db_type is None:
+            return None, None
+        params = []
+        return ' '.join(
+            # This appends to the params being returned.
+            self._iter_column_sql(column_db_type, params, model, field, include_default)
+        ), params
 
     def skip_default(self, field):
         """
@@ -266,6 +282,13 @@ class BaseDatabaseSchemaEditor:
         """
         return False
 
+    def skip_default_on_alter(self, field):
+        """
+        Some backends don't accept default values for certain columns types
+        (i.e. MySQL longtext and longblob) in the ALTER COLUMN statement.
+        """
+        return False
+
     def prepare_default(self, value):
         """
         Only used for backends which have requires_literal_defaults feature
@@ -293,14 +316,15 @@ class BaseDatabaseSchemaEditor:
             else:
                 default = ''
         elif getattr(field, 'auto_now', False) or getattr(field, 'auto_now_add', False):
-            default = datetime.now()
             internal_type = field.get_internal_type()
-            if internal_type == 'DateField':
-                default = default.date()
-            elif internal_type == 'TimeField':
-                default = default.time()
-            elif internal_type == 'DateTimeField':
+            if internal_type == 'DateTimeField':
                 default = timezone.now()
+            else:
+                default = datetime.now()
+                if internal_type == 'DateField':
+                    default = default.date()
+                elif internal_type == 'TimeField':
+                    default = default.time()
         else:
             default = None
         return default
@@ -400,9 +424,9 @@ class BaseDatabaseSchemaEditor:
         for fields in olds.difference(news):
             self._delete_composed_index(model, fields, {'unique': True}, self.sql_delete_unique)
         # Created uniques
-        for fields in news.difference(olds):
-            columns = [model._meta.get_field(field).column for field in fields]
-            self.execute(self._create_unique_sql(model, columns))
+        for field_names in news.difference(olds):
+            fields = [model._meta.get_field(field) for field in field_names]
+            self.execute(self._create_unique_sql(model, fields))
 
     def alter_index_together(self, model, old_index_together, new_index_together):
         """
@@ -508,7 +532,7 @@ class BaseDatabaseSchemaEditor:
         self.execute(sql, params)
         # Drop the default if we need to
         # (Django usually does not use in-database defaults)
-        if not self.skip_default(field) and self.effective_default(field) is not None:
+        if not self.skip_default_on_alter(field) and self.effective_default(field) is not None:
             changes_sql, params = self._alter_column_default_sql(model, None, field, drop=True)
             sql = self.sql_alter_column % {
                 "table": self.quote_name(model._meta.db_table),
@@ -721,7 +745,7 @@ class BaseDatabaseSchemaEditor:
             old_default = self.effective_default(old_field)
             new_default = self.effective_default(new_field)
             if (
-                not self.skip_default(new_field) and
+                not self.skip_default_on_alter(new_field) and
                 old_default != new_default and
                 new_default is not None
             ):
@@ -783,7 +807,7 @@ class BaseDatabaseSchemaEditor:
             self._delete_primary_key(model, strict)
         # Added a unique?
         if self._unique_should_be_added(old_field, new_field):
-            self.execute(self._create_unique_sql(model, [new_field.column]))
+            self.execute(self._create_unique_sql(model, [new_field]))
         # Added an index? Add an index if db_index switched to True or a unique
         # constraint will no longer be used in lieu of an index. The following
         # lines from the truth table show all True cases; the rest are False:
@@ -829,8 +853,8 @@ class BaseDatabaseSchemaEditor:
             self.execute(self._create_fk_sql(model, new_field, "_fk_%(to_table)s_%(to_column)s"))
         # Rebuild FKs that pointed to us if we previously had to drop them
         if drop_foreign_keys:
-            for rel in new_field.model._meta.related_objects:
-                if _is_relevant_relation(rel, new_field) and rel.field.db_constraint:
+            for _, rel in rels_to_update:
+                if rel.field.db_constraint:
                     self.execute(self._create_fk_sql(rel.related_model, rel.field, "_fk"))
         # Does it have check constraints we need to add?
         if old_db_params['check'] != new_db_params['check'] and new_db_params['check']:
@@ -856,8 +880,10 @@ class BaseDatabaseSchemaEditor:
         Return a (sql, params) fragment to set a column to null or non-null
         as required by new_field, or None if no changes are required.
         """
-        if (self.connection.features.interprets_empty_strings_as_nulls and
-                new_field.get_internal_type() in ("CharField", "TextField")):
+        if (
+            self.connection.features.interprets_empty_strings_as_nulls and
+            new_field.empty_strings_allowed
+        ):
             # The field is nullable in the database anyway, leave it alone.
             return
         else:
@@ -934,7 +960,7 @@ class BaseDatabaseSchemaEditor:
             self.sql_alter_column_collate % {
                 'column': self.quote_name(new_field.column),
                 'type': new_type,
-                'collation': self._collate_sql(new_collation) if new_collation else '',
+                'collation': ' ' + self._collate_sql(new_collation) if new_collation else '',
             },
             [],
         )
@@ -1184,16 +1210,16 @@ class BaseDatabaseSchemaEditor:
 
     def _unique_sql(
         self, model, fields, name, condition=None, deferrable=None,
-        include=None, opclasses=None,
+        include=None, opclasses=None, expressions=None,
     ):
         if (
             deferrable and
             not self.connection.features.supports_deferrable_unique_constraints
         ):
             return None
-        if condition or include or opclasses:
-            # Databases support conditional and covering unique constraints via
-            # a unique index.
+        if condition or include or opclasses or expressions:
+            # Databases support conditional, covering, and functional unique
+            # constraints via a unique index.
             sql = self._create_unique_sql(
                 model,
                 fields,
@@ -1201,12 +1227,13 @@ class BaseDatabaseSchemaEditor:
                 condition=condition,
                 include=include,
                 opclasses=opclasses,
+                expressions=expressions,
             )
             if sql:
                 self.deferred_sql.append(sql)
             return None
         constraint = self.sql_unique_constraint % {
-            'columns': ', '.join(map(self.quote_name, fields)),
+            'columns': ', '.join([self.quote_name(field.column) for field in fields]),
             'deferrable': self._deferrable_constraint_sql(deferrable),
         }
         return self.sql_constraint % {
@@ -1215,8 +1242,8 @@ class BaseDatabaseSchemaEditor:
         }
 
     def _create_unique_sql(
-        self, model, columns, name=None, condition=None, deferrable=None,
-        include=None, opclasses=None,
+        self, model, fields, name=None, condition=None, deferrable=None,
+        include=None, opclasses=None, expressions=None,
     ):
         if (
             (
@@ -1224,26 +1251,32 @@ class BaseDatabaseSchemaEditor:
                 not self.connection.features.supports_deferrable_unique_constraints
             ) or
             (condition and not self.connection.features.supports_partial_indexes) or
-            (include and not self.connection.features.supports_covering_indexes)
+            (include and not self.connection.features.supports_covering_indexes) or
+            (expressions and not self.connection.features.supports_expression_indexes)
         ):
             return None
 
         def create_unique_name(*args, **kwargs):
             return self.quote_name(self._create_index_name(*args, **kwargs))
 
-        table = Table(model._meta.db_table, self.quote_name)
+        compiler = Query(model, alias_cols=False).get_compiler(connection=self.connection)
+        table = model._meta.db_table
+        columns = [field.column for field in fields]
         if name is None:
-            name = IndexName(model._meta.db_table, columns, '_uniq', create_unique_name)
+            name = IndexName(table, columns, '_uniq', create_unique_name)
         else:
             name = self.quote_name(name)
-        columns = self._index_columns(table, columns, col_suffixes=(), opclasses=opclasses)
-        if condition or include or opclasses:
+        if condition or include or opclasses or expressions:
             sql = self.sql_create_unique_index
         else:
             sql = self.sql_create_unique
+        if columns:
+            columns = self._index_columns(table, columns, col_suffixes=(), opclasses=opclasses)
+        else:
+            columns = Expressions(table, expressions, compiler, self.quote_value)
         return Statement(
             sql,
-            table=table,
+            table=Table(table, self.quote_name),
             name=name,
             columns=columns,
             condition=self._index_condition_sql(condition),
@@ -1253,7 +1286,7 @@ class BaseDatabaseSchemaEditor:
 
     def _delete_unique_sql(
         self, model, name, condition=None, deferrable=None, include=None,
-        opclasses=None,
+        opclasses=None, expressions=None,
     ):
         if (
             (
@@ -1261,10 +1294,12 @@ class BaseDatabaseSchemaEditor:
                 not self.connection.features.supports_deferrable_unique_constraints
             ) or
             (condition and not self.connection.features.supports_partial_indexes) or
-            (include and not self.connection.features.supports_covering_indexes)
+            (include and not self.connection.features.supports_covering_indexes) or
+            (expressions and not self.connection.features.supports_expression_indexes)
+
         ):
             return None
-        if condition or include or opclasses:
+        if condition or include or opclasses or expressions:
             sql = self.sql_delete_index
         else:
             sql = self.sql_delete_unique
@@ -1348,7 +1383,7 @@ class BaseDatabaseSchemaEditor:
         return self._delete_constraint_sql(self.sql_delete_pk, model, name)
 
     def _collate_sql(self, collation):
-        return ' COLLATE ' + self.quote_name(collation)
+        return 'COLLATE ' + self.quote_name(collation)
 
     def remove_procedure(self, procedure_name, param_types=()):
         sql = self.sql_delete_procedure % {
