@@ -1,13 +1,17 @@
-from __future__ import unicode_literals
-
 from django import forms
 from django.contrib import admin
 from django.contrib.admin import AdminSite
+from django.contrib.auth.backends import ModelBackend
+from django.contrib.auth.middleware import AuthenticationMiddleware
 from django.contrib.contenttypes.admin import GenericStackedInline
+from django.contrib.messages.middleware import MessageMiddleware
+from django.contrib.sessions.middleware import SessionMiddleware
 from django.core import checks
 from django.test import SimpleTestCase, override_settings
 
-from .models import Album, Book, City, Influence, Song, State, TwoAlbumFKAndAnE
+from .models import (
+    Album, Author, Book, City, Influence, Song, State, TwoAlbumFKAndAnE,
+)
 
 
 class SongForm(forms.ModelForm):
@@ -37,13 +41,34 @@ class MyAdmin(admin.ModelAdmin):
         return ['error!']
 
 
+class AuthenticationMiddlewareSubclass(AuthenticationMiddleware):
+    pass
+
+
+class MessageMiddlewareSubclass(MessageMiddleware):
+    pass
+
+
+class ModelBackendSubclass(ModelBackend):
+    pass
+
+
+class SessionMiddlewareSubclass(SessionMiddleware):
+    pass
+
+
 @override_settings(
     SILENCED_SYSTEM_CHECKS=['fields.W342'],  # ForeignKey(unique=True)
-    INSTALLED_APPS=['django.contrib.auth', 'django.contrib.contenttypes', 'admin_checks']
+    INSTALLED_APPS=[
+        'django.contrib.admin',
+        'django.contrib.auth',
+        'django.contrib.contenttypes',
+        'django.contrib.messages',
+        'admin_checks',
+    ],
 )
 class SystemChecksTestCase(SimpleTestCase):
 
-    @override_settings(DEBUG=True)
     def test_checks_are_performed(self):
         admin.site.register(Song, MyAdmin)
         try:
@@ -52,9 +77,171 @@ class SystemChecksTestCase(SimpleTestCase):
             self.assertEqual(errors, expected)
         finally:
             admin.site.unregister(Song)
-            admin.sites.system_check_errors = []
 
-    @override_settings(DEBUG=True)
+    @override_settings(INSTALLED_APPS=['django.contrib.admin'])
+    def test_apps_dependencies(self):
+        errors = admin.checks.check_dependencies()
+        expected = [
+            checks.Error(
+                "'django.contrib.contenttypes' must be in "
+                "INSTALLED_APPS in order to use the admin application.",
+                id="admin.E401",
+            ),
+            checks.Error(
+                "'django.contrib.auth' must be in INSTALLED_APPS in order "
+                "to use the admin application.",
+                id='admin.E405',
+            ),
+            checks.Error(
+                "'django.contrib.messages' must be in INSTALLED_APPS in order "
+                "to use the admin application.",
+                id='admin.E406',
+            ),
+        ]
+        self.assertEqual(errors, expected)
+
+    @override_settings(TEMPLATES=[])
+    def test_no_template_engines(self):
+        self.assertEqual(admin.checks.check_dependencies(), [
+            checks.Error(
+                "A 'django.template.backends.django.DjangoTemplates' "
+                "instance must be configured in TEMPLATES in order to use "
+                "the admin application.",
+                id='admin.E403',
+            )
+        ])
+
+    @override_settings(
+        TEMPLATES=[{
+            'BACKEND': 'django.template.backends.django.DjangoTemplates',
+            'DIRS': [],
+            'APP_DIRS': True,
+            'OPTIONS': {
+                'context_processors': [],
+            },
+        }],
+    )
+    def test_context_processor_dependencies(self):
+        expected = [
+            checks.Error(
+                "'django.contrib.auth.context_processors.auth' must be "
+                "enabled in DjangoTemplates (TEMPLATES) if using the default "
+                "auth backend in order to use the admin application.",
+                id='admin.E402',
+            ),
+            checks.Error(
+                "'django.contrib.messages.context_processors.messages' must "
+                "be enabled in DjangoTemplates (TEMPLATES) in order to use "
+                "the admin application.",
+                id='admin.E404',
+            ),
+            checks.Warning(
+                "'django.template.context_processors.request' must be enabled "
+                "in DjangoTemplates (TEMPLATES) in order to use the admin "
+                "navigation sidebar.",
+                id='admin.W411',
+            )
+        ]
+        self.assertEqual(admin.checks.check_dependencies(), expected)
+        # The first error doesn't happen if
+        # 'django.contrib.auth.backends.ModelBackend' isn't in
+        # AUTHENTICATION_BACKENDS.
+        with self.settings(AUTHENTICATION_BACKENDS=[]):
+            self.assertEqual(admin.checks.check_dependencies(), expected[1:])
+
+    @override_settings(
+        AUTHENTICATION_BACKENDS=['admin_checks.tests.ModelBackendSubclass'],
+        TEMPLATES=[{
+            'BACKEND': 'django.template.backends.django.DjangoTemplates',
+            'DIRS': [],
+            'APP_DIRS': True,
+            'OPTIONS': {
+                'context_processors': [
+                    'django.template.context_processors.request',
+                    'django.contrib.messages.context_processors.messages',
+                ],
+            },
+        }],
+    )
+    def test_context_processor_dependencies_model_backend_subclass(self):
+        self.assertEqual(admin.checks.check_dependencies(), [
+            checks.Error(
+                "'django.contrib.auth.context_processors.auth' must be "
+                "enabled in DjangoTemplates (TEMPLATES) if using the default "
+                "auth backend in order to use the admin application.",
+                id='admin.E402',
+            ),
+        ])
+
+    @override_settings(
+        TEMPLATES=[
+            {
+                'BACKEND': 'django.template.backends.dummy.TemplateStrings',
+                'DIRS': [],
+                'APP_DIRS': True,
+            },
+            {
+                'BACKEND': 'django.template.backends.django.DjangoTemplates',
+                'DIRS': [],
+                'APP_DIRS': True,
+                'OPTIONS': {
+                    'context_processors': [
+                        'django.template.context_processors.request',
+                        'django.contrib.auth.context_processors.auth',
+                        'django.contrib.messages.context_processors.messages',
+                    ],
+                },
+            },
+        ],
+    )
+    def test_several_templates_backends(self):
+        self.assertEqual(admin.checks.check_dependencies(), [])
+
+    @override_settings(MIDDLEWARE=[])
+    def test_middleware_dependencies(self):
+        errors = admin.checks.check_dependencies()
+        expected = [
+            checks.Error(
+                "'django.contrib.auth.middleware.AuthenticationMiddleware' "
+                "must be in MIDDLEWARE in order to use the admin application.",
+                id='admin.E408',
+            ),
+            checks.Error(
+                "'django.contrib.messages.middleware.MessageMiddleware' "
+                "must be in MIDDLEWARE in order to use the admin application.",
+                id='admin.E409',
+            ),
+            checks.Error(
+                "'django.contrib.sessions.middleware.SessionMiddleware' "
+                "must be in MIDDLEWARE in order to use the admin application.",
+                hint=(
+                    "Insert "
+                    "'django.contrib.sessions.middleware.SessionMiddleware' "
+                    "before "
+                    "'django.contrib.auth.middleware.AuthenticationMiddleware'."
+                ),
+                id='admin.E410',
+            ),
+        ]
+        self.assertEqual(errors, expected)
+
+    @override_settings(MIDDLEWARE=[
+        'admin_checks.tests.AuthenticationMiddlewareSubclass',
+        'admin_checks.tests.MessageMiddlewareSubclass',
+        'admin_checks.tests.SessionMiddlewareSubclass',
+    ])
+    def test_middleware_subclasses(self):
+        self.assertEqual(admin.checks.check_dependencies(), [])
+
+    @override_settings(MIDDLEWARE=[
+        'django.contrib.does.not.Exist',
+        'django.contrib.auth.middleware.AuthenticationMiddleware',
+        'django.contrib.messages.middleware.MessageMiddleware',
+        'django.contrib.sessions.middleware.SessionMiddleware',
+    ])
+    def test_admin_check_ignores_import_error_in_middleware(self):
+        self.assertEqual(admin.checks.check_dependencies(), [])
+
     def test_custom_adminsite(self):
         class CustomAdminSite(admin.AdminSite):
             pass
@@ -67,7 +254,26 @@ class SystemChecksTestCase(SimpleTestCase):
             self.assertEqual(errors, expected)
         finally:
             custom_site.unregister(Song)
-            admin.sites.system_check_errors = []
+
+    def test_allows_checks_relying_on_other_modeladmins(self):
+        class MyBookAdmin(admin.ModelAdmin):
+            def check(self, **kwargs):
+                errors = super().check(**kwargs)
+                author_admin = self.admin_site._registry.get(Author)
+                if author_admin is None:
+                    errors.append('AuthorAdmin missing!')
+                return errors
+
+        class MyAuthorAdmin(admin.ModelAdmin):
+            pass
+
+        admin.site.register(Book, MyBookAdmin)
+        admin.site.register(Author, MyAuthorAdmin)
+        try:
+            self.assertEqual(admin.site.check(None), [])
+        finally:
+            admin.site.unregister(Book)
+            admin.site.unregister(Author)
 
     def test_field_name_not_in_list_display(self):
         class SongAdmin(admin.ModelAdmin):
@@ -78,13 +284,37 @@ class SystemChecksTestCase(SimpleTestCase):
             checks.Error(
                 "The value of 'list_editable[0]' refers to 'original_release', "
                 "which is not contained in 'list_display'.",
-                hint=None,
                 obj=SongAdmin,
                 id='admin.E122',
             )
         ]
         self.assertEqual(errors, expected)
 
+    def test_list_editable_not_a_list_or_tuple(self):
+        class SongAdmin(admin.ModelAdmin):
+            list_editable = 'test'
+
+        self.assertEqual(SongAdmin(Song, AdminSite()).check(), [
+            checks.Error(
+                "The value of 'list_editable' must be a list or tuple.",
+                obj=SongAdmin,
+                id='admin.E120',
+            )
+        ])
+
+    def test_list_editable_missing_field(self):
+        class SongAdmin(admin.ModelAdmin):
+            list_editable = ('test',)
+
+        self.assertEqual(SongAdmin(Song, AdminSite()).check(), [
+            checks.Error(
+                "The value of 'list_editable[0]' refers to 'test', which is "
+                "not a field of 'admin_checks.Song'.",
+                obj=SongAdmin,
+                id='admin.E121',
+            )
+        ])
+
     def test_readonly_and_editable(self):
         class SongAdmin(admin.ModelAdmin):
             readonly_fields = ["original_release"]
@@ -98,9 +328,8 @@ class SystemChecksTestCase(SimpleTestCase):
         errors = SongAdmin(Song, AdminSite()).check()
         expected = [
             checks.Error(
-                ("The value of 'list_editable[0]' refers to 'original_release', "
-                 "which is not editable through the admin."),
-                hint=None,
+                "The value of 'list_editable[0]' refers to 'original_release', "
+                "which is not editable through the admin.",
                 obj=SongAdmin,
                 id='admin.E125',
             )
@@ -129,16 +358,15 @@ class SystemChecksTestCase(SimpleTestCase):
 
     def test_custom_get_form_with_fieldsets(self):
         """
-        Ensure that the fieldsets checks are skipped when the ModelAdmin.get_form() method
+        The fieldsets checks are skipped when the ModelAdmin.get_form() method
         is overridden.
-        Refs #19445.
         """
         errors = ValidFormFieldsets(Song, AdminSite()).check()
         self.assertEqual(errors, [])
 
     def test_fieldsets_fields_non_tuple(self):
         """
-        Tests for a tuple/list for the first fieldset's fields.
+        The first fieldset's fields must be a list/tuple.
         """
         class NotATupleAdmin(admin.ModelAdmin):
             list_display = ["pk", "title"]
@@ -153,7 +381,6 @@ class SystemChecksTestCase(SimpleTestCase):
         expected = [
             checks.Error(
                 "The value of 'fieldsets[0][1]['fields']' must be a list or tuple.",
-                hint=None,
                 obj=NotATupleAdmin,
                 id='admin.E008',
             )
@@ -162,7 +389,7 @@ class SystemChecksTestCase(SimpleTestCase):
 
     def test_nonfirst_fieldset(self):
         """
-        Tests for a tuple/list for the second fieldset's fields.
+        The second fieldset's fields must be a list/tuple.
         """
         class NotATupleAdmin(admin.ModelAdmin):
             fieldsets = [
@@ -178,7 +405,6 @@ class SystemChecksTestCase(SimpleTestCase):
         expected = [
             checks.Error(
                 "The value of 'fieldsets[1][1]['fields']' must be a list or tuple.",
-                hint=None,
                 obj=NotATupleAdmin,
                 id='admin.E008',
             )
@@ -196,7 +422,6 @@ class SystemChecksTestCase(SimpleTestCase):
         expected = [
             checks.Error(
                 "The value of 'exclude' must be a list or tuple.",
-                hint=None,
                 obj=ExcludedFields1,
                 id='admin.E014',
             )
@@ -211,7 +436,6 @@ class SystemChecksTestCase(SimpleTestCase):
         expected = [
             checks.Error(
                 "The value of 'exclude' contains duplicate field(s).",
-                hint=None,
                 obj=ExcludedFields2,
                 id='admin.E015',
             )
@@ -231,7 +455,6 @@ class SystemChecksTestCase(SimpleTestCase):
         expected = [
             checks.Error(
                 "The value of 'exclude' must be a list or tuple.",
-                hint=None,
                 obj=ExcludedFieldsInline,
                 id='admin.E014',
             )
@@ -254,9 +477,8 @@ class SystemChecksTestCase(SimpleTestCase):
         errors = AlbumAdmin(Album, AdminSite()).check()
         expected = [
             checks.Error(
-                ("Cannot exclude the field 'album', because it is the foreign key "
-                 "to the parent model 'admin_checks.Album'."),
-                hint=None,
+                "Cannot exclude the field 'album', because it is the foreign key "
+                "to the parent model 'admin_checks.Album'.",
                 obj=SongInline,
                 id='admin.E201',
             )
@@ -279,8 +501,8 @@ class SystemChecksTestCase(SimpleTestCase):
 
     def test_generic_inline_model_admin_non_generic_model(self):
         """
-        Ensure that a model without a GenericForeignKey raises problems if it's included
-        in an GenericInlineModelAdmin definition.
+        A model without a GenericForeignKey raises problems if it's included
+        in a GenericInlineModelAdmin definition.
         """
         class BookInline(GenericStackedInline):
             model = Book
@@ -292,7 +514,6 @@ class SystemChecksTestCase(SimpleTestCase):
         expected = [
             checks.Error(
                 "'admin_checks.Book' has no GenericForeignKey.",
-                hint=None,
                 obj=BookInline,
                 id='admin.E301',
             )
@@ -300,7 +521,10 @@ class SystemChecksTestCase(SimpleTestCase):
         self.assertEqual(errors, expected)
 
     def test_generic_inline_model_admin_bad_ct_field(self):
-        "A GenericInlineModelAdmin raises problems if the ct_field points to a non-existent field."
+        """
+        A GenericInlineModelAdmin errors if the ct_field points to a
+        nonexistent field.
+        """
         class InfluenceInline(GenericStackedInline):
             model = Influence
             ct_field = 'nonexistent'
@@ -312,7 +536,6 @@ class SystemChecksTestCase(SimpleTestCase):
         expected = [
             checks.Error(
                 "'ct_field' references 'nonexistent', which is not a field on 'admin_checks.Influence'.",
-                hint=None,
                 obj=InfluenceInline,
                 id='admin.E302',
             )
@@ -320,7 +543,10 @@ class SystemChecksTestCase(SimpleTestCase):
         self.assertEqual(errors, expected)
 
     def test_generic_inline_model_admin_bad_fk_field(self):
-        "A GenericInlineModelAdmin raises problems if the ct_fk_field points to a non-existent field."
+        """
+        A GenericInlineModelAdmin errors if the ct_fk_field points to a
+        nonexistent field.
+        """
         class InfluenceInline(GenericStackedInline):
             model = Influence
             ct_fk_field = 'nonexistent'
@@ -332,7 +558,6 @@ class SystemChecksTestCase(SimpleTestCase):
         expected = [
             checks.Error(
                 "'ct_fk_field' references 'nonexistent', which is not a field on 'admin_checks.Influence'.",
-                hint=None,
                 obj=InfluenceInline,
                 id='admin.E303',
             )
@@ -356,7 +581,6 @@ class SystemChecksTestCase(SimpleTestCase):
             checks.Error(
                 "'admin_checks.Influence' has no GenericForeignKey using "
                 "content type field 'name' and object ID field 'object_id'.",
-                hint=None,
                 obj=InfluenceInline,
                 id='admin.E304',
             )
@@ -380,7 +604,6 @@ class SystemChecksTestCase(SimpleTestCase):
             checks.Error(
                 "'admin_checks.Influence' has no GenericForeignKey using "
                 "content type field 'content_type' and object ID field 'name'.",
-                hint=None,
                 obj=InfluenceInline,
                 id='admin.E304',
             )
@@ -388,19 +611,15 @@ class SystemChecksTestCase(SimpleTestCase):
         self.assertEqual(errors, expected)
 
     def test_app_label_in_admin_checks(self):
-        """
-        Regression test for #15669 - Include app label in admin system check messages
-        """
-        class RawIdNonexistingAdmin(admin.ModelAdmin):
-            raw_id_fields = ('nonexisting',)
+        class RawIdNonexistentAdmin(admin.ModelAdmin):
+            raw_id_fields = ('nonexistent',)
 
-        errors = RawIdNonexistingAdmin(Album, AdminSite()).check()
+        errors = RawIdNonexistentAdmin(Album, AdminSite()).check()
         expected = [
             checks.Error(
-                "The value of 'raw_id_fields[0]' refers to 'nonexisting', "
-                "which is not an attribute of 'admin_checks.Album'.",
-                hint=None,
-                obj=RawIdNonexistingAdmin,
+                "The value of 'raw_id_fields[0]' refers to 'nonexistent', "
+                "which is not a field of 'admin_checks.Album'.",
+                obj=RawIdNonexistentAdmin,
                 id='admin.E002',
             )
         ]
@@ -433,8 +652,9 @@ class SystemChecksTestCase(SimpleTestCase):
         errors = MyAdmin(Album, AdminSite()).check()
         expected = [
             checks.Error(
-                "'admin_checks.TwoAlbumFKAndAnE' has more than one ForeignKey to 'admin_checks.Album'.",
-                hint=None,
+                "'admin_checks.TwoAlbumFKAndAnE' has more than one ForeignKey "
+                "to 'admin_checks.Album'. You must specify a 'fk_name' "
+                "attribute.",
                 obj=TwoAlbumFKAndAnEInline,
                 id='admin.E202',
             )
@@ -452,6 +672,18 @@ class SystemChecksTestCase(SimpleTestCase):
         errors = MyAdmin(Album, AdminSite()).check()
         self.assertEqual(errors, [])
 
+    def test_inlines_property(self):
+        class CitiesInline(admin.TabularInline):
+            model = City
+
+        class StateAdmin(admin.ModelAdmin):
+            @property
+            def inlines(self):
+                return [CitiesInline]
+
+        errors = StateAdmin(State, AdminSite()).check()
+        self.assertEqual(errors, [])
+
     def test_readonly(self):
         class SongAdmin(admin.ModelAdmin):
             readonly_fields = ("title",)
@@ -460,6 +692,7 @@ class SystemChecksTestCase(SimpleTestCase):
         self.assertEqual(errors, [])
 
     def test_readonly_on_method(self):
+        @admin.display
         def my_function(obj):
             pass
 
@@ -473,6 +706,7 @@ class SystemChecksTestCase(SimpleTestCase):
         class SongAdmin(admin.ModelAdmin):
             readonly_fields = ("readonly_method_on_modeladmin",)
 
+            @admin.display
             def readonly_method_on_modeladmin(self, obj):
                 pass
 
@@ -485,6 +719,7 @@ class SystemChecksTestCase(SimpleTestCase):
 
             def __getattr__(self, item):
                 if item == "dynamic_method":
+                    @admin.display
                     def method(obj):
                         pass
                     return method
@@ -507,9 +742,8 @@ class SystemChecksTestCase(SimpleTestCase):
         errors = SongAdmin(Song, AdminSite()).check()
         expected = [
             checks.Error(
-                ("The value of 'readonly_fields[1]' is not a callable, an attribute "
-                 "of 'SongAdmin', or an attribute of 'admin_checks.Song'."),
-                hint=None,
+                "The value of 'readonly_fields[1]' is not a callable, an attribute "
+                "of 'SongAdmin', or an attribute of 'admin_checks.Song'.",
                 obj=SongAdmin,
                 id='admin.E035',
             )
@@ -524,17 +758,29 @@ class SystemChecksTestCase(SimpleTestCase):
         errors = CityInline(State, AdminSite()).check()
         expected = [
             checks.Error(
-                ("The value of 'readonly_fields[0]' is not a callable, an attribute "
-                 "of 'CityInline', or an attribute of 'admin_checks.City'."),
-                hint=None,
+                "The value of 'readonly_fields[0]' is not a callable, an attribute "
+                "of 'CityInline', or an attribute of 'admin_checks.City'.",
                 obj=CityInline,
                 id='admin.E035',
             )
         ]
         self.assertEqual(errors, expected)
 
+    def test_readonly_fields_not_list_or_tuple(self):
+        class SongAdmin(admin.ModelAdmin):
+            readonly_fields = 'test'
+
+        self.assertEqual(SongAdmin(Song, AdminSite()).check(), [
+            checks.Error(
+                "The value of 'readonly_fields' must be a list or tuple.",
+                obj=SongAdmin,
+                id='admin.E034',
+            )
+        ])
+
     def test_extra(self):
         class SongAdmin(admin.ModelAdmin):
+            @admin.display
             def awesome_song(self, instance):
                 if instance.title == "Born to Run":
                     return "Best Ever!"
@@ -562,9 +808,8 @@ class SystemChecksTestCase(SimpleTestCase):
         errors = BookAdmin(Book, AdminSite()).check()
         expected = [
             checks.Error(
-                ("The value of 'fields' cannot include the ManyToManyField 'authors', "
-                 "because that field manually specifies a relationship model."),
-                hint=None,
+                "The value of 'fields' cannot include the ManyToManyField 'authors', "
+                "because that field manually specifies a relationship model.",
                 obj=BookAdmin,
                 id='admin.E013',
             )
@@ -581,9 +826,8 @@ class SystemChecksTestCase(SimpleTestCase):
         errors = FieldsetBookAdmin(Book, AdminSite()).check()
         expected = [
             checks.Error(
-                ("The value of 'fieldsets[1][1][\"fields\"]' cannot include the ManyToManyField "
-                 "'authors', because that field manually specifies a relationship model."),
-                hint=None,
+                "The value of 'fieldsets[1][1][\"fields\"]' cannot include the ManyToManyField "
+                "'authors', because that field manually specifies a relationship model.",
                 obj=FieldsetBookAdmin,
                 id='admin.E013',
             )
@@ -663,7 +907,6 @@ class SystemChecksTestCase(SimpleTestCase):
         expected = [
             checks.Error(
                 "The value of 'fields' contains duplicate field(s).",
-                hint=None,
                 obj=MyModelAdmin,
                 id='admin.E006'
             )
@@ -682,7 +925,6 @@ class SystemChecksTestCase(SimpleTestCase):
         expected = [
             checks.Error(
                 "There are duplicate field(s) in 'fieldsets[0][1]'.",
-                hint=None,
                 obj=MyModelAdmin,
                 id='admin.E012'
             )
