@@ -294,7 +294,45 @@ class SomeChildModel(models.Model):
     parent = models.ForeignKey(SomeParentModel, models.CASCADE)
     readonly_field = models.CharField(max_length=1)
 
+
+# Models for #30231
+class Course(models.Model):
+    name = models.CharField(max_length=128)
+
+    def __str__(self):
+        return self.name
+
+
+class Class(models.Model):
+    person = models.ManyToManyField(Person, verbose_name='attendant')
+    course = models.ForeignKey(Course, on_delete=models.CASCADE)
+
+
+class CourseProxy(Course):
+
+    class Meta:
+        proxy = True
+
+
+class CourseProxy1(Course):
+
+    class Meta:
+        proxy = True
+
+
+class CourseProxy2(Course):
+
+    class Meta:
+        proxy = True
+
+
 # Other models
+class ShowInlineParent(models.Model):
+    show_inlines = models.BooleanField(default=False)
+
+
+class ShowInlineChild(models.Model):
+    parent = models.ForeignKey(ShowInlineParent, on_delete=models.CASCADE)
 
 
 class ProfileCollection(models.Model):
@@ -305,3 +343,19 @@ class Profile(models.Model):
     collection = models.ForeignKey(ProfileCollection, models.SET_NULL, blank=True, null=True)
     first_name = models.CharField(max_length=100)
     last_name = models.CharField(max_length=100)
+
+
+class VerboseNameProfile(Profile):
+    class Meta:
+        verbose_name = 'Model with verbose name only'
+
+
+class VerboseNamePluralProfile(Profile):
+    class Meta:
+        verbose_name_plural = 'Model with verbose name plural only'
+
+
+class BothVerboseNameProfile(Profile):
+    class Meta:
+        verbose_name = 'Model with both - name'
+        verbose_name_plural = 'Model with both - plural name'
