@@ -4,8 +4,8 @@ from decimal import Decimal
 from django.core.exceptions import FieldDoesNotExist, FieldError
 from django.db.models import (
     BooleanField, Case, CharField, Count, DateTimeField, DecimalField, Exists,
-    ExpressionWrapper, F, FloatField, Func, IntegerField, Max,
-    NullBooleanField, OuterRef, Q, Subquery, Sum, Value, When,
+    ExpressionWrapper, F, FloatField, Func, IntegerField, Max, OuterRef, Q,
+    Subquery, Sum, Value, When,
 )
 from django.db.models.expressions import RawSQL
 from django.db.models.functions import (
@@ -210,6 +210,12 @@ class NonAggregateAnnotationTestCase(TestCase):
         self.assertEqual(len(books), Book.objects.count())
         self.assertTrue(all(not book.selected for book in books))
 
+    def test_empty_queryset_annotation(self):
+        qs = Author.objects.annotate(
+            empty=Subquery(Author.objects.values('id').none())
+        )
+        self.assertIsNone(qs.first().empty)
+
     def test_annotate_with_aggregation(self):
         books = Book.objects.annotate(is_book=Value(1), rating_count=Count('rating'))
         for book in books:
@@ -241,7 +247,7 @@ class NonAggregateAnnotationTestCase(TestCase):
             ),
             rating_count=Count('rating'),
         ).first()
-        self.assertEqual(book.isnull_pubdate, False)
+        self.assertIs(book.isnull_pubdate, False)
         self.assertEqual(book.rating_count, 1)
 
     @skipUnlessDBFeature('supports_boolean_expr_in_select_clause')
@@ -641,14 +647,12 @@ class NonAggregateAnnotationTestCase(TestCase):
             is_book=Value(True, output_field=BooleanField()),
             is_pony=Value(False, output_field=BooleanField()),
             is_none=Value(None, output_field=BooleanField(null=True)),
-            is_none_old=Value(None, output_field=NullBooleanField()),
         )
         self.assertGreater(len(books), 0)
         for book in books:
             self.assertIs(book.is_book, True)
             self.assertIs(book.is_pony, False)
             self.assertIsNone(book.is_none)
-            self.assertIsNone(book.is_none_old)
 
     def test_annotation_in_f_grouped_by_annotation(self):
         qs = (
@@ -766,40 +770,6 @@ class NonAggregateAnnotationTestCase(TestCase):
             {'name': 'Wesley J. Chun', 'max_pages': 0},
         ])
 
-    def test_alias_sql_injection(self):
-        crafted_alias = """injected_name" from "annotations_book"; --"""
-        msg = (
-            "Column aliases cannot contain whitespace characters, quotation marks, "
-            "semicolons, or SQL comments."
-        )
-        with self.assertRaisesMessage(ValueError, msg):
-            Book.objects.annotate(**{crafted_alias: Value(1)})
-
-    def test_alias_forbidden_chars(self):
-        tests = [
-            'al"ias',
-            "a'lias",
-            "ali`as",
-            "alia s",
-            "alias\t",
-            "ali\nas",
-            "alias--",
-            "ali/*as",
-            "alias*/",
-            "alias;",
-            # [] are used by MSSQL.
-            "alias[",
-            "alias]",
-        ]
-        msg = (
-            "Column aliases cannot contain whitespace characters, quotation marks, "
-            "semicolons, or SQL comments."
-        )
-        for crafted_alias in tests:
-            with self.subTest(crafted_alias):
-                with self.assertRaisesMessage(ValueError, msg):
-                    Book.objects.annotate(**{crafted_alias: Value(1)})
-
 
 class AliasTests(TestCase):
     @classmethod
@@ -1030,12 +1000,3 @@ class AliasTests(TestCase):
             with self.subTest(operation=operation):
                 with self.assertRaisesMessage(FieldError, msg):
                     getattr(qs, operation)('rating_alias')
-
-    def test_alias_sql_injection(self):
-        crafted_alias = """injected_name" from "annotations_book"; --"""
-        msg = (
-            "Column aliases cannot contain whitespace characters, quotation marks, "
-            "semicolons, or SQL comments."
-        )
-        with self.assertRaisesMessage(ValueError, msg):
-            Book.objects.alias(**{crafted_alias: Value(1)})
