@@ -102,6 +102,7 @@ class MockInputTests(TestCase):
 
 
 class GetDefaultUsernameTestCase(TestCase):
+    databases = {'default', 'other'}
 
     def setUp(self):
         self.old_get_system_username = management.get_system_username
@@ -128,6 +129,15 @@ class GetDefaultUsernameTestCase(TestCase):
         management.get_system_username = lambda: 'J\xfalia'
         self.assertEqual(management.get_default_username(), 'julia')
 
+    def test_with_database(self):
+        User.objects.create(username='joe')
+        management.get_system_username = lambda: 'joe'
+        self.assertEqual(management.get_default_username(), '')
+        self.assertEqual(management.get_default_username(database='other'), 'joe')
+
+        User.objects.using('other').create(username='joe')
+        self.assertEqual(management.get_default_username(database='other'), '')
+
 
 @override_settings(AUTH_PASSWORD_VALIDATORS=[
     {'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator'},
@@ -310,13 +320,12 @@ class CreatesuperuserManagementCommandTestCase(TestCase):
         self.assertFalse(u.has_usable_password())
 
     def test_email_in_username(self):
-        new_io = StringIO()
         call_command(
             "createsuperuser",
             interactive=False,
             username="joe+admin@somewhere.org",
             email="joe@somewhere.org",
-            stdout=new_io
+            verbosity=0,
         )
         u = User._default_manager.get(username="joe+admin@somewhere.org")
         self.assertEqual(u.email, 'joe@somewhere.org')
@@ -496,6 +505,55 @@ class CreatesuperuserManagementCommandTestCase(TestCase):
 
         test(self)
 
+    @override_settings(AUTH_USER_MODEL='auth_tests.CustomUserWithFK')
+    def test_fields_with_fk_via_option_interactive(self):
+        new_io = StringIO()
+        group = Group.objects.create(name='mygroup')
+        email = Email.objects.create(email='mymail@gmail.com')
+
+        @mock_inputs({'password': 'nopasswd'})
+        def test(self):
+            call_command(
+                'createsuperuser',
+                interactive=True,
+                username=email.pk,
+                email=email.email,
+                group=group.pk,
+                stdout=new_io,
+                stdin=MockTTY(),
+            )
+
+            command_output = new_io.getvalue().strip()
+            self.assertEqual(command_output, 'Superuser created successfully.')
+            u = CustomUserWithFK._default_manager.get(email=email)
+            self.assertEqual(u.username, email)
+            self.assertEqual(u.group, group)
+
+        test(self)
+
+    @override_settings(AUTH_USER_MODEL='auth_tests.CustomUserWithFK')
+    def test_validate_fk_via_option_interactive(self):
+        email = Email.objects.create(email='mymail@gmail.com')
+        Group.objects.all().delete()
+        nonexistent_group_id = 1
+        msg = f'group instance with id {nonexistent_group_id} does not exist.'
+
+        @mock_inputs({
+            'password': 'nopasswd',
+            'Username (Email.id): ': email.pk,
+            'Email (Email.email): ': email.email,
+        })
+        def test(self):
+            with self.assertRaisesMessage(CommandError, msg):
+                call_command(
+                    'createsuperuser',
+                    group=nonexistent_group_id,
+                    stdin=MockTTY(),
+                    verbosity=0,
+                )
+
+        test(self)
+
     @override_settings(AUTH_USER_MODEL='auth_tests.CustomUserWithM2m')
     def test_fields_with_m2m(self):
         new_io = StringIO()
@@ -704,6 +762,46 @@ class CreatesuperuserManagementCommandTestCase(TestCase):
 
         test(self)
 
+    @override_settings(
+        AUTH_USER_MODEL='auth_tests.CustomUser',
+        AUTH_PASSWORD_VALIDATORS=[
+            {'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator'},
+        ]
+    )
+    def test_validate_password_against_required_fields_via_option(self):
+        new_io = StringIO()
+        first_name = 'josephine'
+        entered_passwords = [
+            first_name, first_name,
+            'superduperunguessablepassword', 'superduperunguessablepassword',
+        ]
+
+        def bad_then_good_password():
+            return entered_passwords.pop(0)
+
+        @mock_inputs({
+            'password': bad_then_good_password,
+            'bypass': 'n',
+        })
+        def test(self):
+            call_command(
+                'createsuperuser',
+                interactive=True,
+                first_name=first_name,
+                date_of_birth='1970-01-01',
+                email='joey@example.com',
+                stdin=MockTTY(),
+                stdout=new_io,
+                stderr=new_io,
+            )
+            self.assertEqual(
+                new_io.getvalue().strip(),
+                'The password is too similar to the first name.\n'
+                'Superuser created successfully.'
+            )
+
+        test(self)
+
     def test_blank_username(self):
         """Creation fails if --username is blank."""
         new_io = StringIO()
@@ -985,6 +1083,25 @@ class CreatesuperuserManagementCommandTestCase(TestCase):
         # Environment variables are ignored for non-required fields.
         self.assertEqual(user.first_name, '')
 
+    @override_settings(AUTH_USER_MODEL='auth_tests.CustomUserWithM2m')
+    def test_environment_variable_m2m_non_interactive(self):
+        new_io = StringIO()
+        org_id_1 = Organization.objects.create(name='Organization 1').pk
+        org_id_2 = Organization.objects.create(name='Organization 2').pk
+        with mock.patch.dict(os.environ, {
+            'DJANGO_SUPERUSER_ORGS': f'{org_id_1},{org_id_2}',
+        }):
+            call_command(
+                'createsuperuser',
+                interactive=False,
+                username='joe',
+                stdout=new_io,
+            )
+        command_output = new_io.getvalue().strip()
+        self.assertEqual(command_output, 'Superuser created successfully.')
+        user = CustomUserWithM2M._default_manager.get(username='joe')
+        self.assertEqual(user.orgs.count(), 2)
+
     @mock.patch.dict(os.environ, {
         'DJANGO_SUPERUSER_USERNAME': 'test_superuser',
         'DJANGO_SUPERUSER_EMAIL': 'joe@somewhere.org',
@@ -1032,7 +1149,7 @@ class MultiDBCreatesuperuserTestCase(TestCase):
 
     def test_createsuperuser_command_with_database_option(self):
         """
-        changepassword --database should operate on the specified DB.
+        createsuperuser --database should operate on the specified DB.
         """
         new_io = StringIO()
         call_command(
@@ -1048,6 +1165,36 @@ class MultiDBCreatesuperuserTestCase(TestCase):
         user = User.objects.using('other').get(username='joe')
         self.assertEqual(user.email, 'joe@somewhere.org')
 
+    def test_createsuperuser_command_suggested_username_with_database_option(self):
+        default_username = get_default_username(database='other')
+        qs = User.objects.using('other')
+
+        @mock_inputs({'password': 'nopasswd', 'username': '', 'email': ''})
+        def test_other_create_with_suggested_username(self):
+            call_command(
+                'createsuperuser',
+                interactive=True,
+                stdin=MockTTY(),
+                verbosity=0,
+                database='other',
+            )
+            self.assertIs(qs.filter(username=default_username).exists(), True)
+
+        test_other_create_with_suggested_username(self)
+
+        @mock_inputs({'password': 'nopasswd', 'Username: ': 'other', 'email': ''})
+        def test_other_no_suggestion(self):
+            call_command(
+                'createsuperuser',
+                interactive=True,
+                stdin=MockTTY(),
+                verbosity=0,
+                database='other',
+            )
+            self.assertIs(qs.filter(username='other').exists(), True)
+
+        test_other_no_suggestion(self)
+
 
 class CreatePermissionsTests(TestCase):
 
@@ -1092,7 +1239,7 @@ class CreatePermissionsTests(TestCase):
         with self.assertNumQueries(0):
             create_permissions(self.app_config, verbosity=0, apps=state.apps)
         # Unavailable auth.Permission
-        state = migrations.state.ProjectState(real_apps=['contenttypes'])
+        state = migrations.state.ProjectState(real_apps={'contenttypes'})
         with self.assertNumQueries(0):
             create_permissions(self.app_config, verbosity=0, apps=state.apps)
 
