@@ -1,24 +1,23 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 import sys
+from io import StringIO
 
 from django.apps import apps
 from django.core import checks
 from django.core.checks import Error, Warning
+from django.core.checks.messages import CheckMessage
 from django.core.checks.registry import CheckRegistry
 from django.core.management import call_command
 from django.core.management.base import CommandError
 from django.db import models
 from django.test import SimpleTestCase
-from django.test.utils import override_settings, override_system_checks
-from django.utils.encoding import force_text
-from django.utils.six import StringIO
+from django.test.utils import (
+    isolate_apps, override_settings, override_system_checks,
+)
 
-from .models import SimpleModel
+from .models import SimpleModel, my_check
 
 
-class DummyObj(object):
+class DummyObj:
     def __repr__(self):
         return "obj"
 
@@ -32,10 +31,10 @@ class SystemCheckFrameworkTests(SimpleTestCase):
             return [1, 2, 3]
 
         def f2(**kwargs):
-            return [4, ]
+            return [4]
 
         def f3(**kwargs):
-            return [5, ]
+            return [5]
 
         calls = [0]
 
@@ -68,45 +67,90 @@ class SystemCheckFrameworkTests(SimpleTestCase):
         self.assertEqual(errors, errors2)
         self.assertEqual(sorted(errors), [4, 5])
 
+    def test_register_no_kwargs_error(self):
+        registry = CheckRegistry()
+        msg = 'Check functions must accept keyword arguments (**kwargs).'
+        with self.assertRaisesMessage(TypeError, msg):
+            @registry.register
+            def no_kwargs(app_configs, databases):
+                pass
+
+    def test_register_run_checks_non_iterable(self):
+        registry = CheckRegistry()
+
+        @registry.register
+        def return_non_iterable(**kwargs):
+            return Error('Message')
+
+        msg = (
+            'The function %r did not return a list. All functions registered '
+            'with the checks registry must return a list.' % return_non_iterable
+        )
+        with self.assertRaisesMessage(TypeError, msg):
+            registry.run_checks()
+
 
 class MessageTests(SimpleTestCase):
 
     def test_printing(self):
         e = Error("Message", hint="Hint", obj=DummyObj())
         expected = "obj: Message\n\tHINT: Hint"
-        self.assertEqual(force_text(e), expected)
+        self.assertEqual(str(e), expected)
 
     def test_printing_no_hint(self):
-        e = Error("Message", hint=None, obj=DummyObj())
+        e = Error("Message", obj=DummyObj())
         expected = "obj: Message"
-        self.assertEqual(force_text(e), expected)
+        self.assertEqual(str(e), expected)
 
     def test_printing_no_object(self):
-        e = Error("Message", hint="Hint", obj=None)
+        e = Error("Message", hint="Hint")
         expected = "?: Message\n\tHINT: Hint"
-        self.assertEqual(force_text(e), expected)
+        self.assertEqual(str(e), expected)
 
     def test_printing_with_given_id(self):
         e = Error("Message", hint="Hint", obj=DummyObj(), id="ID")
         expected = "obj: (ID) Message\n\tHINT: Hint"
-        self.assertEqual(force_text(e), expected)
+        self.assertEqual(str(e), expected)
 
     def test_printing_field_error(self):
         field = SimpleModel._meta.get_field('field')
-        e = Error("Error", hint=None, obj=field)
+        e = Error("Error", obj=field)
         expected = "check_framework.SimpleModel.field: Error"
-        self.assertEqual(force_text(e), expected)
+        self.assertEqual(str(e), expected)
 
     def test_printing_model_error(self):
-        e = Error("Error", hint=None, obj=SimpleModel)
+        e = Error("Error", obj=SimpleModel)
         expected = "check_framework.SimpleModel: Error"
-        self.assertEqual(force_text(e), expected)
+        self.assertEqual(str(e), expected)
 
     def test_printing_manager_error(self):
         manager = SimpleModel.manager
-        e = Error("Error", hint=None, obj=manager)
+        e = Error("Error", obj=manager)
         expected = "check_framework.SimpleModel.manager: Error"
-        self.assertEqual(force_text(e), expected)
+        self.assertEqual(str(e), expected)
+
+    def test_equal_to_self(self):
+        e = Error("Error", obj=SimpleModel)
+        self.assertEqual(e, e)
+
+    def test_equal_to_same_constructed_check(self):
+        e1 = Error("Error", obj=SimpleModel)
+        e2 = Error("Error", obj=SimpleModel)
+        self.assertEqual(e1, e2)
+
+    def test_not_equal_to_different_constructed_check(self):
+        e1 = Error("Error", obj=SimpleModel)
+        e2 = Error("Error2", obj=SimpleModel)
+        self.assertNotEqual(e1, e2)
+
+    def test_not_equal_to_non_check(self):
+        e = Error("Error", obj=DummyObj())
+        self.assertNotEqual(e, 'a string')
+
+    def test_invalid_level(self):
+        msg = 'The first argument should be level.'
+        with self.assertRaisesMessage(TypeError, msg):
+            CheckMessage('ERROR', 'Message')
 
 
 def simple_system_check(**kwargs):
@@ -116,13 +160,17 @@ def simple_system_check(**kwargs):
 
 def tagged_system_check(**kwargs):
     tagged_system_check.kwargs = kwargs
-    return []
+    return [checks.Warning('System Check')]
+
+
 tagged_system_check.tags = ['simpletag']
 
 
 def deployment_system_check(**kwargs):
     deployment_system_check.kwargs = kwargs
     return [checks.Warning('Deployment Check')]
+
+
 deployment_system_check.tags = ['deploymenttag']
 
 
@@ -140,26 +188,28 @@ class CheckCommandTests(SimpleTestCase):
     @override_system_checks([simple_system_check, tagged_system_check])
     def test_simple_call(self):
         call_command('check')
-        self.assertEqual(simple_system_check.kwargs, {'app_configs': None})
-        self.assertEqual(tagged_system_check.kwargs, {'app_configs': None})
+        self.assertEqual(simple_system_check.kwargs, {'app_configs': None, 'databases': None})
+        self.assertEqual(tagged_system_check.kwargs, {'app_configs': None, 'databases': None})
 
     @override_system_checks([simple_system_check, tagged_system_check])
     def test_given_app(self):
         call_command('check', 'auth', 'admin')
         auth_config = apps.get_app_config('auth')
         admin_config = apps.get_app_config('admin')
-        self.assertEqual(simple_system_check.kwargs, {'app_configs': [auth_config, admin_config]})
-        self.assertEqual(tagged_system_check.kwargs, {'app_configs': [auth_config, admin_config]})
+        self.assertEqual(simple_system_check.kwargs, {'app_configs': [auth_config, admin_config], 'databases': None})
+        self.assertEqual(tagged_system_check.kwargs, {'app_configs': [auth_config, admin_config], 'databases': None})
 
     @override_system_checks([simple_system_check, tagged_system_check])
     def test_given_tag(self):
         call_command('check', tags=['simpletag'])
-        self.assertEqual(simple_system_check.kwargs, None)
-        self.assertEqual(tagged_system_check.kwargs, {'app_configs': None})
+        self.assertIsNone(simple_system_check.kwargs)
+        self.assertEqual(tagged_system_check.kwargs, {'app_configs': None, 'databases': None})
 
     @override_system_checks([simple_system_check, tagged_system_check])
     def test_invalid_tag(self):
-        self.assertRaises(CommandError, call_command, 'check', tags=['missingtag'])
+        msg = 'There is no system check with the "missingtag" tag.'
+        with self.assertRaisesMessage(CommandError, msg):
+            call_command('check', tags=['missingtag'])
 
     @override_system_checks([simple_system_check])
     def test_list_tags_empty(self):
@@ -192,25 +242,18 @@ class CheckCommandTests(SimpleTestCase):
         call_command('check', deploy=True, tags=['deploymenttag'])
         self.assertIn('Deployment Check', sys.stderr.getvalue())
 
+    @override_system_checks([tagged_system_check])
+    def test_fail_level(self):
+        with self.assertRaises(CommandError):
+            call_command('check', fail_level='WARNING')
+
 
 def custom_error_system_check(app_configs, **kwargs):
-    return [
-        Error(
-            'Error',
-            hint=None,
-            id='myerrorcheck.E001',
-        )
-    ]
+    return [Error('Error', id='myerrorcheck.E001')]
 
 
 def custom_warning_system_check(app_configs, **kwargs):
-    return [
-        Warning(
-            'Warning',
-            hint=None,
-            id='mywarningcheck.E001',
-        )
-    ]
+    return [Warning('Warning', id='mywarningcheck.E001')]
 
 
 class SilencingCheckTests(SimpleTestCase):
@@ -228,10 +271,7 @@ class SilencingCheckTests(SimpleTestCase):
     def test_silenced_error(self):
         out = StringIO()
         err = StringIO()
-        try:
-            call_command('check', stdout=out, stderr=err)
-        except CommandError:
-            self.fail("The mycheck.E001 check should be silenced.")
+        call_command('check', stdout=out, stderr=err)
         self.assertEqual(out.getvalue(), 'System check identified no issues (1 silenced).\n')
         self.assertEqual(err.getvalue(), '')
 
@@ -240,32 +280,15 @@ class SilencingCheckTests(SimpleTestCase):
     def test_silenced_warning(self):
         out = StringIO()
         err = StringIO()
-        try:
-            call_command('check', stdout=out, stderr=err)
-        except CommandError:
-            self.fail("The mycheck.E001 check should be silenced.")
-
+        call_command('check', stdout=out, stderr=err)
         self.assertEqual(out.getvalue(), 'System check identified no issues (1 silenced).\n')
         self.assertEqual(err.getvalue(), '')
 
 
-class IsolateModelsMixin(object):
-    def setUp(self):
-        self.current_models = apps.all_models[__package__]
-        self.saved_models = set(self.current_models)
-
-    def tearDown(self):
-        for model in (set(self.current_models) - self.saved_models):
-            del self.current_models[model]
-        apps.clear_cache()
-
-
-class CheckFrameworkReservedNamesTests(IsolateModelsMixin, SimpleTestCase):
-    @override_settings(
-        SILENCED_SYSTEM_CHECKS=['models.E20', 'fields.W342'],  # ForeignKey(unique=True)
-        INSTALLED_APPS=['django.contrib.auth', 'django.contrib.contenttypes', 'check_framework']
-    )
-    def test_model_check_method_not_shadowed(self):
+class CheckFrameworkReservedNamesTests(SimpleTestCase):
+    @isolate_apps('check_framework', kwarg_name='apps')
+    @override_system_checks([checks.model_checks.check_all_models])
+    def test_model_check_method_not_shadowed(self, apps):
         class ModelWithAttributeCalledCheck(models.Model):
             check = 42
 
@@ -283,28 +306,36 @@ class CheckFrameworkReservedNamesTests(IsolateModelsMixin, SimpleTestCase):
                 related_name='check',
             )
 
-        errors = checks.run_checks()
+        errors = checks.run_checks(app_configs=apps.get_app_configs())
         expected = [
             Error(
                 "The 'ModelWithAttributeCalledCheck.check()' class method is "
                 "currently overridden by 42.",
-                hint=None,
                 obj=ModelWithAttributeCalledCheck,
                 id='models.E020'
             ),
+            Error(
+                "The 'ModelWithFieldCalledCheck.check()' class method is "
+                "currently overridden by %r." % ModelWithFieldCalledCheck.check,
+                obj=ModelWithFieldCalledCheck,
+                id='models.E020'
+            ),
             Error(
                 "The 'ModelWithRelatedManagerCalledCheck.check()' class method is "
                 "currently overridden by %r." % ModelWithRelatedManagerCalledCheck.check,
-                hint=None,
                 obj=ModelWithRelatedManagerCalledCheck,
                 id='models.E020'
             ),
             Error(
                 "The 'ModelWithDescriptorCalledCheck.check()' class method is "
                 "currently overridden by %r." % ModelWithDescriptorCalledCheck.check,
-                hint=None,
                 obj=ModelWithDescriptorCalledCheck,
                 id='models.E020'
             ),
         ]
         self.assertEqual(errors, expected)
+
+
+class ChecksRunDuringTests(SimpleTestCase):
+    def test_registered_check_did_run(self):
+        self.assertTrue(my_check.did_run)
