@@ -2,25 +2,32 @@ import datetime
 import pickle
 import unittest
 import uuid
+from collections import namedtuple
 from copy import deepcopy
+from decimal import Decimal
 from unittest import mock
 
 from django.core.exceptions import FieldError
 from django.db import DatabaseError, NotSupportedError, connection
 from django.db.models import (
-    Avg, BooleanField, Case, CharField, Count, DateField, DateTimeField,
-    DurationField, Exists, Expression, ExpressionList, ExpressionWrapper, F,
-    Func, IntegerField, Max, Min, Model, OrderBy, OuterRef, Q, StdDev,
-    Subquery, Sum, TimeField, UUIDField, Value, Variance, When,
+    AutoField, Avg, BinaryField, BooleanField, Case, CharField, Count,
+    DateField, DateTimeField, DecimalField, DurationField, Exists, Expression,
+    ExpressionList, ExpressionWrapper, F, FloatField, Func, IntegerField, Max,
+    Min, Model, OrderBy, OuterRef, Q, StdDev, Subquery, Sum, TimeField,
+    UUIDField, Value, Variance, When,
+)
+from django.db.models.expressions import (
+    Col, Combinable, CombinedExpression, RawSQL, Ref,
 )
-from django.db.models.expressions import Col, Combinable, Random, RawSQL, Ref
 from django.db.models.functions import (
     Coalesce, Concat, Left, Length, Lower, Substr, Upper,
 )
 from django.db.models.sql import constants
 from django.db.models.sql.datastructures import Join
 from django.test import SimpleTestCase, TestCase, skipUnlessDBFeature
-from django.test.utils import Approximate, isolate_apps
+from django.test.utils import (
+    Approximate, CaptureQueriesContext, isolate_apps, register_lookup,
+)
 from django.utils.functional import SimpleLazyObject
 
 from .models import (
@@ -65,9 +72,9 @@ class BasicExpressionsTests(TestCase):
         companies = Company.objects.annotate(
             foo=RawSQL('%s', ['value']),
         ).filter(foo='value').order_by('name')
-        self.assertQuerysetEqual(
+        self.assertSequenceEqual(
             companies,
-            ['<Company: Example Inc.>', '<Company: Foobar Ltd.>', '<Company: Test GmbH>'],
+            [self.example_inc, self.foobar_ltd, self.gmbh],
         )
 
     def test_annotate_values_count(self):
@@ -337,10 +344,10 @@ class BasicExpressionsTests(TestCase):
 
     def test_ticket_11722_iexact_lookup(self):
         Employee.objects.create(firstname="John", lastname="Doe")
-        Employee.objects.create(firstname="Test", lastname="test")
+        test = Employee.objects.create(firstname="Test", lastname="test")
 
         queryset = Employee.objects.filter(firstname__iexact=F('lastname'))
-        self.assertQuerysetEqual(queryset, ["<Employee: Test test>"])
+        self.assertSequenceEqual(queryset, [test])
 
     def test_ticket_16731_startswith_lookup(self):
         Employee.objects.create(firstname="John", lastname="Doe")
@@ -663,6 +670,18 @@ class BasicExpressionsTests(TestCase):
         # contain nested aggregates.
         self.assertNotIn('GROUP BY', sql)
 
+    @skipUnlessDBFeature('supports_over_clause')
+    def test_aggregate_rawsql_annotation(self):
+        with self.assertNumQueries(1) as ctx:
+            aggregate = Company.objects.annotate(
+                salary=RawSQL('SUM(num_chairs) OVER (ORDER BY num_employees)', []),
+            ).aggregate(
+                count=Count('pk'),
+            )
+            self.assertEqual(aggregate, {'count': 3})
+        sql = ctx.captured_queries[0]['sql']
+        self.assertNotIn('GROUP BY', sql)
+
     def test_explicit_output_field(self):
         class FuncA(Func):
             output_field = CharField()
@@ -710,8 +729,24 @@ class BasicExpressionsTests(TestCase):
         ).get(pk=self.gmbh.pk)
         self.assertEqual(gmbh_salary.max_ceo_salary_raise, 2332)
 
+    def test_annotation_with_nested_outerref(self):
+        self.gmbh.point_of_contact = Employee.objects.get(lastname='Meyer')
+        self.gmbh.save()
+        inner = Employee.objects.annotate(
+            outer_lastname=OuterRef(OuterRef('lastname')),
+        ).filter(lastname__startswith=Left('outer_lastname', 1))
+        qs = Employee.objects.annotate(
+            ceo_company=Subquery(
+                Company.objects.filter(
+                    point_of_contact__in=inner,
+                    ceo__pk=OuterRef('pk'),
+                ).values('name'),
+            ),
+        ).filter(ceo_company__isnull=False)
+        self.assertEqual(qs.get().ceo_company, 'Test GmbH')
+
     def test_pickle_expression(self):
-        expr = Value(1, output_field=IntegerField())
+        expr = Value(1)
         expr.convert_value  # populate cached property
         self.assertEqual(pickle.loads(pickle.dumps(expr)), expr)
 
@@ -757,29 +792,61 @@ class BasicExpressionsTests(TestCase):
                 output_field=BooleanField(),
             ),
         )
-        self.assertSequenceEqual(qs, [self.example_inc.ceo, self.foobar_ltd.ceo, self.max])
+        self.assertCountEqual(qs, [self.example_inc.ceo, self.foobar_ltd.ceo, self.max])
 
     def test_boolean_expression_combined(self):
         is_ceo = Company.objects.filter(ceo=OuterRef('pk'))
         is_poc = Company.objects.filter(point_of_contact=OuterRef('pk'))
         self.gmbh.point_of_contact = self.max
         self.gmbh.save()
-        self.assertSequenceEqual(
+        self.assertCountEqual(
             Employee.objects.filter(Exists(is_ceo) | Exists(is_poc)),
             [self.example_inc.ceo, self.foobar_ltd.ceo, self.max],
         )
-        self.assertSequenceEqual(
+        self.assertCountEqual(
             Employee.objects.filter(Exists(is_ceo) & Exists(is_poc)),
             [self.max],
         )
-        self.assertSequenceEqual(
+        self.assertCountEqual(
             Employee.objects.filter(Exists(is_ceo) & Q(salary__gte=30)),
             [self.max],
         )
-        self.assertSequenceEqual(
+        self.assertCountEqual(
             Employee.objects.filter(Exists(is_poc) | Q(salary__lt=15)),
             [self.example_inc.ceo, self.max],
         )
+        self.assertCountEqual(
+            Employee.objects.filter(Q(salary__gte=30) & Exists(is_ceo)),
+            [self.max],
+        )
+        self.assertCountEqual(
+            Employee.objects.filter(Q(salary__lt=15) | Exists(is_poc)),
+            [self.example_inc.ceo, self.max],
+        )
+
+    def test_boolean_expression_combined_with_empty_Q(self):
+        is_poc = Company.objects.filter(point_of_contact=OuterRef('pk'))
+        self.gmbh.point_of_contact = self.max
+        self.gmbh.save()
+        tests = [
+            Exists(is_poc) & Q(),
+            Q() & Exists(is_poc),
+            Exists(is_poc) | Q(),
+            Q() | Exists(is_poc),
+            Q(Exists(is_poc)) & Q(),
+            Q() & Q(Exists(is_poc)),
+            Q(Exists(is_poc)) | Q(),
+            Q() | Q(Exists(is_poc)),
+        ]
+        for conditions in tests:
+            with self.subTest(conditions):
+                self.assertCountEqual(Employee.objects.filter(conditions), [self.max])
+
+    def test_boolean_expression_in_Q(self):
+        is_poc = Company.objects.filter(point_of_contact=OuterRef('pk'))
+        self.gmbh.point_of_contact = self.max
+        self.gmbh.save()
+        self.assertCountEqual(Employee.objects.filter(Q(Exists(is_poc))), [self.max])
 
 
 class IterableLookupInnerExpressionsTests(TestCase):
@@ -789,44 +856,38 @@ class IterableLookupInnerExpressionsTests(TestCase):
         # MySQL requires that the values calculated for expressions don't pass
         # outside of the field's range, so it's inconvenient to use the values
         # in the more general tests.
-        Company.objects.create(name='5020 Ltd', num_employees=50, num_chairs=20, ceo=ceo)
-        Company.objects.create(name='5040 Ltd', num_employees=50, num_chairs=40, ceo=ceo)
-        Company.objects.create(name='5050 Ltd', num_employees=50, num_chairs=50, ceo=ceo)
-        Company.objects.create(name='5060 Ltd', num_employees=50, num_chairs=60, ceo=ceo)
-        Company.objects.create(name='99300 Ltd', num_employees=99, num_chairs=300, ceo=ceo)
+        cls.c5020 = Company.objects.create(name='5020 Ltd', num_employees=50, num_chairs=20, ceo=ceo)
+        cls.c5040 = Company.objects.create(name='5040 Ltd', num_employees=50, num_chairs=40, ceo=ceo)
+        cls.c5050 = Company.objects.create(name='5050 Ltd', num_employees=50, num_chairs=50, ceo=ceo)
+        cls.c5060 = Company.objects.create(name='5060 Ltd', num_employees=50, num_chairs=60, ceo=ceo)
+        cls.c99300 = Company.objects.create(name='99300 Ltd', num_employees=99, num_chairs=300, ceo=ceo)
 
     def test_in_lookup_allows_F_expressions_and_expressions_for_integers(self):
         # __in lookups can use F() expressions for integers.
         queryset = Company.objects.filter(num_employees__in=([F('num_chairs') - 10]))
-        self.assertQuerysetEqual(queryset, ['<Company: 5060 Ltd>'], ordered=False)
-        self.assertQuerysetEqual(
+        self.assertSequenceEqual(queryset, [self.c5060])
+        self.assertCountEqual(
             Company.objects.filter(num_employees__in=([F('num_chairs') - 10, F('num_chairs') + 10])),
-            ['<Company: 5040 Ltd>', '<Company: 5060 Ltd>'],
-            ordered=False
+            [self.c5040, self.c5060],
         )
-        self.assertQuerysetEqual(
+        self.assertCountEqual(
             Company.objects.filter(
                 num_employees__in=([F('num_chairs') - 10, F('num_chairs'), F('num_chairs') + 10])
             ),
-            ['<Company: 5040 Ltd>', '<Company: 5050 Ltd>', '<Company: 5060 Ltd>'],
-            ordered=False
+            [self.c5040, self.c5050, self.c5060],
         )
 
     def test_expressions_in_lookups_join_choice(self):
         midpoint = datetime.time(13, 0)
         t1 = Time.objects.create(time=datetime.time(12, 0))
         t2 = Time.objects.create(time=datetime.time(14, 0))
-        SimulationRun.objects.create(start=t1, end=t2, midpoint=midpoint)
+        s1 = SimulationRun.objects.create(start=t1, end=t2, midpoint=midpoint)
         SimulationRun.objects.create(start=t1, end=None, midpoint=midpoint)
         SimulationRun.objects.create(start=None, end=t2, midpoint=midpoint)
         SimulationRun.objects.create(start=None, end=None, midpoint=midpoint)
 
         queryset = SimulationRun.objects.filter(midpoint__range=[F('start__time'), F('end__time')])
-        self.assertQuerysetEqual(
-            queryset,
-            ['<SimulationRun: 13:00:00 (12:00:00 to 14:00:00)>'],
-            ordered=False
-        )
+        self.assertSequenceEqual(queryset, [s1])
         for alias in queryset.query.alias_map.values():
             if isinstance(alias, Join):
                 self.assertEqual(alias.join_type, constants.INNER)
@@ -840,30 +901,30 @@ class IterableLookupInnerExpressionsTests(TestCase):
     def test_range_lookup_allows_F_expressions_and_expressions_for_integers(self):
         # Range lookups can use F() expressions for integers.
         Company.objects.filter(num_employees__exact=F("num_chairs"))
-        self.assertQuerysetEqual(
+        self.assertCountEqual(
             Company.objects.filter(num_employees__range=(F('num_chairs'), 100)),
-            ['<Company: 5020 Ltd>', '<Company: 5040 Ltd>', '<Company: 5050 Ltd>'],
-            ordered=False
+            [self.c5020, self.c5040, self.c5050],
         )
-        self.assertQuerysetEqual(
+        self.assertCountEqual(
             Company.objects.filter(num_employees__range=(F('num_chairs') - 10, F('num_chairs') + 10)),
-            ['<Company: 5040 Ltd>', '<Company: 5050 Ltd>', '<Company: 5060 Ltd>'],
-            ordered=False
+            [self.c5040, self.c5050, self.c5060],
         )
-        self.assertQuerysetEqual(
+        self.assertCountEqual(
             Company.objects.filter(num_employees__range=(F('num_chairs') - 10, 100)),
-            ['<Company: 5020 Ltd>', '<Company: 5040 Ltd>', '<Company: 5050 Ltd>', '<Company: 5060 Ltd>'],
-            ordered=False
+            [self.c5020, self.c5040, self.c5050, self.c5060],
         )
-        self.assertQuerysetEqual(
+        self.assertCountEqual(
             Company.objects.filter(num_employees__range=(1, 100)),
-            [
-                '<Company: 5020 Ltd>', '<Company: 5040 Ltd>', '<Company: 5050 Ltd>',
-                '<Company: 5060 Ltd>', '<Company: 99300 Ltd>',
-            ],
-            ordered=False
+            [self.c5020, self.c5040, self.c5050, self.c5060, self.c99300],
         )
 
+    def test_range_lookup_namedtuple(self):
+        EmployeeRange = namedtuple('EmployeeRange', ['minimum', 'maximum'])
+        qs = Company.objects.filter(
+            num_employees__range=EmployeeRange(minimum=51, maximum=100),
+        )
+        self.assertSequenceEqual(qs, [self.c99300])
+
     @unittest.skipUnless(connection.vendor == 'sqlite',
                          "This defensive test only works on databases that don't validate parameter types")
     def test_complex_expressions_do_not_introduce_sql_injection_via_untrusted_string_inclusion(self):
@@ -898,7 +959,7 @@ class IterableLookupInnerExpressionsTests(TestCase):
             completed=end.date(),
             estimated_time=end - start,
         )
-        Result.objects.create(
+        r1 = Result.objects.create(
             experiment=experiment_1,
             result_time=datetime.datetime(2016, 2, 4, 15, 0, 0),
         )
@@ -913,11 +974,11 @@ class IterableLookupInnerExpressionsTests(TestCase):
 
         within_experiment_time = [F('experiment__start'), F('experiment__end')]
         queryset = Result.objects.filter(result_time__range=within_experiment_time)
-        self.assertQuerysetEqual(queryset, ["<Result: Result at 2016-02-04 15:00:00>"])
+        self.assertSequenceEqual(queryset, [r1])
 
         within_experiment_time = [F('experiment__start'), F('experiment__end')]
         queryset = Result.objects.filter(result_time__range=within_experiment_time)
-        self.assertQuerysetEqual(queryset, ["<Result: Result at 2016-02-04 15:00:00>"])
+        self.assertSequenceEqual(queryset, [r1])
 
 
 class FTests(SimpleTestCase):
@@ -977,30 +1038,27 @@ class ExpressionsTests(TestCase):
         refs #16731
         """
         Employee.objects.bulk_create([
-            Employee(firstname="%Joh\\nny", lastname="%Joh\\n"),
             Employee(firstname="Johnny", lastname="%John"),
             Employee(firstname="Jean-Claude", lastname="Claud_"),
-            Employee(firstname="Jean-Claude", lastname="Claude"),
             Employee(firstname="Jean-Claude", lastname="Claude%"),
             Employee(firstname="Johnny", lastname="Joh\\n"),
-            Employee(firstname="Johnny", lastname="John"),
             Employee(firstname="Johnny", lastname="_ohn"),
         ])
+        claude = Employee.objects.create(firstname='Jean-Claude', lastname='Claude')
+        john = Employee.objects.create(firstname='Johnny', lastname='John')
+        john_sign = Employee.objects.create(firstname='%Joh\\nny', lastname='%Joh\\n')
 
-        self.assertQuerysetEqual(
+        self.assertCountEqual(
             Employee.objects.filter(firstname__contains=F('lastname')),
-            ["<Employee: %Joh\\nny %Joh\\n>", "<Employee: Jean-Claude Claude>", "<Employee: Johnny John>"],
-            ordered=False,
+            [john_sign, john, claude],
         )
-        self.assertQuerysetEqual(
+        self.assertCountEqual(
             Employee.objects.filter(firstname__startswith=F('lastname')),
-            ["<Employee: %Joh\\nny %Joh\\n>", "<Employee: Johnny John>"],
-            ordered=False,
+            [john_sign, john],
         )
-        self.assertQuerysetEqual(
+        self.assertSequenceEqual(
             Employee.objects.filter(firstname__endswith=F('lastname')),
-            ["<Employee: Jean-Claude Claude>"],
-            ordered=False,
+            [claude],
         )
 
     def test_insensitive_patterns_escape(self):
@@ -1010,30 +1068,27 @@ class ExpressionsTests(TestCase):
         expression -- refs #16731
         """
         Employee.objects.bulk_create([
-            Employee(firstname="%Joh\\nny", lastname="%joh\\n"),
             Employee(firstname="Johnny", lastname="%john"),
             Employee(firstname="Jean-Claude", lastname="claud_"),
-            Employee(firstname="Jean-Claude", lastname="claude"),
             Employee(firstname="Jean-Claude", lastname="claude%"),
             Employee(firstname="Johnny", lastname="joh\\n"),
-            Employee(firstname="Johnny", lastname="john"),
             Employee(firstname="Johnny", lastname="_ohn"),
         ])
+        claude = Employee.objects.create(firstname='Jean-Claude', lastname='claude')
+        john = Employee.objects.create(firstname='Johnny', lastname='john')
+        john_sign = Employee.objects.create(firstname='%Joh\\nny', lastname='%joh\\n')
 
-        self.assertQuerysetEqual(
+        self.assertCountEqual(
             Employee.objects.filter(firstname__icontains=F('lastname')),
-            ["<Employee: %Joh\\nny %joh\\n>", "<Employee: Jean-Claude claude>", "<Employee: Johnny john>"],
-            ordered=False,
+            [john_sign, john, claude],
         )
-        self.assertQuerysetEqual(
+        self.assertCountEqual(
             Employee.objects.filter(firstname__istartswith=F('lastname')),
-            ["<Employee: %Joh\\nny %joh\\n>", "<Employee: Johnny john>"],
-            ordered=False,
+            [john_sign, john],
         )
-        self.assertQuerysetEqual(
+        self.assertSequenceEqual(
             Employee.objects.filter(firstname__iendswith=F('lastname')),
-            ["<Employee: Jean-Claude claude>"],
-            ordered=False,
+            [claude],
         )
 
 
@@ -1098,7 +1153,8 @@ class ExpressionsNumericTests(TestCase):
         """
         self.assertQuerysetEqual(
             Number.objects.all(),
-            ['<Number: -1, -1.000>', '<Number: 42, 42.000>', '<Number: 1337, 1337.000>'],
+            [(-1, -1), (42, 42), (1337, 1337)],
+            lambda n: (n.integer, round(n.float)),
             ordered=False
         )
 
@@ -1109,7 +1165,8 @@ class ExpressionsNumericTests(TestCase):
         self.assertEqual(Number.objects.filter(integer__gt=0).update(integer=F('integer') + 1), 2)
         self.assertQuerysetEqual(
             Number.objects.all(),
-            ['<Number: -1, -1.000>', '<Number: 43, 42.000>', '<Number: 1338, 1337.000>'],
+            [(-1, -1), (43, 42), (1338, 1337)],
+            lambda n: (n.integer, round(n.float)),
             ordered=False
         )
 
@@ -1121,7 +1178,8 @@ class ExpressionsNumericTests(TestCase):
         self.assertEqual(Number.objects.filter(integer__gt=0).update(integer=F('integer') + 1), 2)
         self.assertQuerysetEqual(
             Number.objects.exclude(float=F('integer')),
-            ['<Number: 43, 42.000>', '<Number: 1338, 1337.000>'],
+            [(43, 42), (1338, 1337)],
+            lambda n: (n.integer, round(n.float)),
             ordered=False
         )
 
@@ -1136,6 +1194,13 @@ class ExpressionsNumericTests(TestCase):
         self.assertEqual(Number.objects.get(pk=n.pk).integer, 10)
         self.assertEqual(Number.objects.get(pk=n.pk).float, Approximate(256.900, places=3))
 
+    def test_decimal_expression(self):
+        n = Number.objects.create(integer=1, decimal_value=Decimal('0.5'))
+        n.decimal_value = F('decimal_value') - Decimal('0.4')
+        n.save()
+        n.refresh_from_db()
+        self.assertEqual(n.decimal_value, Decimal('0.1'))
+
 
 class ExpressionOperatorTests(TestCase):
     @classmethod
@@ -1204,19 +1269,23 @@ class ExpressionOperatorTests(TestCase):
         self.assertEqual(Number.objects.get(pk=self.n.pk).integer, 58)
         self.assertEqual(Number.objects.get(pk=self.n1.pk).integer, -10)
 
+    def test_lefthand_transformed_field_bitwise_or(self):
+        Employee.objects.create(firstname='Max', lastname='Mustermann')
+        with register_lookup(CharField, Length):
+            qs = Employee.objects.annotate(bitor=F('lastname__length').bitor(48))
+            self.assertEqual(qs.get().bitor, 58)
+
     def test_lefthand_power(self):
         # LH Power arithmetic operation on floats and integers
         Number.objects.filter(pk=self.n.pk).update(integer=F('integer') ** 2, float=F('float') ** 1.5)
         self.assertEqual(Number.objects.get(pk=self.n.pk).integer, 1764)
         self.assertEqual(Number.objects.get(pk=self.n.pk).float, Approximate(61.02, places=2))
 
-    @unittest.skipIf(connection.vendor == 'oracle', "Oracle doesn't support bitwise XOR.")
     def test_lefthand_bitwise_xor(self):
         Number.objects.update(integer=F('integer').bitxor(48))
         self.assertEqual(Number.objects.get(pk=self.n.pk).integer, 26)
         self.assertEqual(Number.objects.get(pk=self.n1.pk).integer, -26)
 
-    @unittest.skipIf(connection.vendor == 'oracle', "Oracle doesn't support bitwise XOR.")
     def test_lefthand_bitwise_xor_null(self):
         employee = Employee.objects.create(firstname='John', lastname='Doe')
         Employee.objects.update(salary=F('salary').bitxor(48))
@@ -1468,12 +1537,46 @@ class FTimeDeltaTests(TestCase):
         ))
         self.assertIsNone(queryset.first().shifted)
 
+    def test_durationfield_multiply_divide(self):
+        Experiment.objects.update(scalar=2)
+        tests = [
+            (Decimal('2'), 2),
+            (F('scalar'), 2),
+            (2, 2),
+            (3.2, 3.2),
+        ]
+        for expr, scalar in tests:
+            with self.subTest(expr=expr):
+                qs = Experiment.objects.annotate(
+                    multiplied=ExpressionWrapper(
+                        expr * F('estimated_time'),
+                        output_field=DurationField(),
+                    ),
+                    divided=ExpressionWrapper(
+                        F('estimated_time') / expr,
+                        output_field=DurationField(),
+                    ),
+                )
+                for experiment in qs:
+                    self.assertEqual(
+                        experiment.multiplied,
+                        experiment.estimated_time * scalar,
+                    )
+                    self.assertEqual(
+                        experiment.divided,
+                        experiment.estimated_time / scalar,
+                    )
+
+    def test_duration_expressions(self):
+        for delta in self.deltas:
+            qs = Experiment.objects.annotate(duration=F('estimated_time') + delta)
+            for obj in qs:
+                self.assertEqual(obj.duration, obj.estimated_time + delta)
+
     @skipUnlessDBFeature('supports_temporal_subtraction')
     def test_date_subtraction(self):
         queryset = Experiment.objects.annotate(
-            completion_duration=ExpressionWrapper(
-                F('completed') - F('assigned'), output_field=DurationField()
-            )
+            completion_duration=F('completed') - F('assigned'),
         )
 
         at_least_5_days = {e.name for e in queryset.filter(completion_duration__gte=datetime.timedelta(days=5))}
@@ -1485,10 +1588,9 @@ class FTimeDeltaTests(TestCase):
         less_than_5_days = {e.name for e in queryset.filter(completion_duration__lt=datetime.timedelta(days=5))}
         self.assertEqual(less_than_5_days, {'e0', 'e1', 'e2'})
 
-        queryset = Experiment.objects.annotate(difference=ExpressionWrapper(
-            F('completed') - Value(None, output_field=DateField()),
-            output_field=DurationField(),
-        ))
+        queryset = Experiment.objects.annotate(
+            difference=F('completed') - Value(None, output_field=DateField()),
+        )
         self.assertIsNone(queryset.first().difference)
 
         queryset = Experiment.objects.annotate(shifted=ExpressionWrapper(
@@ -1501,9 +1603,7 @@ class FTimeDeltaTests(TestCase):
     def test_date_subquery_subtraction(self):
         subquery = Experiment.objects.filter(pk=OuterRef('pk')).values('completed')
         queryset = Experiment.objects.annotate(
-            difference=ExpressionWrapper(
-                subquery - F('completed'), output_field=DurationField(),
-            ),
+            difference=subquery - F('completed'),
         ).filter(difference=datetime.timedelta())
         self.assertTrue(queryset.exists())
 
@@ -1518,9 +1618,7 @@ class FTimeDeltaTests(TestCase):
                 self.e0.completed,
                 output_field=DateField(),
             ),
-            difference=ExpressionWrapper(
-                F('date_case') - F('completed_value'), output_field=DurationField(),
-            ),
+            difference=F('date_case') - F('completed_value'),
         ).filter(difference=datetime.timedelta())
         self.assertEqual(queryset.get(), self.e0)
 
@@ -1528,20 +1626,16 @@ class FTimeDeltaTests(TestCase):
     def test_time_subtraction(self):
         Time.objects.create(time=datetime.time(12, 30, 15, 2345))
         queryset = Time.objects.annotate(
-            difference=ExpressionWrapper(
-                F('time') - Value(datetime.time(11, 15, 0), output_field=TimeField()),
-                output_field=DurationField(),
-            )
+            difference=F('time') - Value(datetime.time(11, 15, 0)),
         )
         self.assertEqual(
             queryset.get().difference,
             datetime.timedelta(hours=1, minutes=15, seconds=15, microseconds=2345)
         )
 
-        queryset = Time.objects.annotate(difference=ExpressionWrapper(
-            F('time') - Value(None, output_field=TimeField()),
-            output_field=DurationField(),
-        ))
+        queryset = Time.objects.annotate(
+            difference=F('time') - Value(None, output_field=TimeField()),
+        )
         self.assertIsNone(queryset.first().difference)
 
         queryset = Time.objects.annotate(shifted=ExpressionWrapper(
@@ -1555,9 +1649,7 @@ class FTimeDeltaTests(TestCase):
         Time.objects.create(time=datetime.time(12, 30, 15, 2345))
         subquery = Time.objects.filter(pk=OuterRef('pk')).values('time')
         queryset = Time.objects.annotate(
-            difference=ExpressionWrapper(
-                subquery - F('time'), output_field=DurationField(),
-            ),
+            difference=subquery - F('time'),
         ).filter(difference=datetime.timedelta())
         self.assertTrue(queryset.exists())
 
@@ -1573,10 +1665,9 @@ class FTimeDeltaTests(TestCase):
         ]
         self.assertEqual(over_estimate, ['e4'])
 
-        queryset = Experiment.objects.annotate(difference=ExpressionWrapper(
-            F('start') - Value(None, output_field=DateTimeField()),
-            output_field=DurationField(),
-        ))
+        queryset = Experiment.objects.annotate(
+            difference=F('start') - Value(None, output_field=DateTimeField()),
+        )
         self.assertIsNone(queryset.first().difference)
 
         queryset = Experiment.objects.annotate(shifted=ExpressionWrapper(
@@ -1589,9 +1680,7 @@ class FTimeDeltaTests(TestCase):
     def test_datetime_subquery_subtraction(self):
         subquery = Experiment.objects.filter(pk=OuterRef('pk')).values('start')
         queryset = Experiment.objects.annotate(
-            difference=ExpressionWrapper(
-                subquery - F('start'), output_field=DurationField(),
-            ),
+            difference=subquery - F('start'),
         ).filter(difference=datetime.timedelta())
         self.assertTrue(queryset.exists())
 
@@ -1599,9 +1688,7 @@ class FTimeDeltaTests(TestCase):
     def test_datetime_subtraction_microseconds(self):
         delta = datetime.timedelta(microseconds=8999999999999999)
         Experiment.objects.update(end=F('start') + delta)
-        qs = Experiment.objects.annotate(
-            delta=ExpressionWrapper(F('end') - F('start'), output_field=DurationField())
-        )
+        qs = Experiment.objects.annotate(delta=F('end') - F('start'))
         for e in qs:
             self.assertEqual(e.delta, delta)
 
@@ -1625,7 +1712,7 @@ class FTimeDeltaTests(TestCase):
 
     def test_date_minus_duration(self):
         more_than_4_days = Experiment.objects.filter(
-            assigned__lt=F('completed') - Value(datetime.timedelta(days=4), output_field=DurationField())
+            assigned__lt=F('completed') - Value(datetime.timedelta(days=4))
         )
         self.assertQuerysetEqual(more_than_4_days, ['e3', 'e4', 'e5'], lambda e: e.name)
 
@@ -1675,6 +1762,22 @@ class ValueTests(TestCase):
         self.assertEqual(len(kwargs), 1)
         self.assertEqual(kwargs['output_field'].deconstruct(), CharField().deconstruct())
 
+    def test_repr(self):
+        tests = [
+            (None, 'Value(None)'),
+            ('str', "Value('str')"),
+            (True, 'Value(True)'),
+            (42, 'Value(42)'),
+            (
+                datetime.datetime(2019, 5, 15),
+                'Value(datetime.datetime(2019, 5, 15, 0, 0))',
+            ),
+            (Decimal('3.14'), "Value(Decimal('3.14'))"),
+        ]
+        for value, expected in tests:
+            with self.subTest(value=value):
+                self.assertEqual(repr(Value(value)), expected)
+
     def test_equal(self):
         value = Value('name')
         self.assertEqual(value, Value('name'))
@@ -1699,6 +1802,63 @@ class ValueTests(TestCase):
         with self.assertRaisesMessage(ValueError, msg):
             ExpressionList()
 
+    def test_compile_unresolved(self):
+        # This test might need to be revisited later on if #25425 is enforced.
+        compiler = Time.objects.all().query.get_compiler(connection=connection)
+        value = Value('foo')
+        self.assertEqual(value.as_sql(compiler, connection), ('%s', ['foo']))
+        value = Value('foo', output_field=CharField())
+        self.assertEqual(value.as_sql(compiler, connection), ('%s', ['foo']))
+
+    def test_output_field_decimalfield(self):
+        Time.objects.create()
+        time = Time.objects.annotate(one=Value(1, output_field=DecimalField())).first()
+        self.assertEqual(time.one, 1)
+
+    def test_resolve_output_field(self):
+        value_types = [
+            ('str', CharField),
+            (True, BooleanField),
+            (42, IntegerField),
+            (3.14, FloatField),
+            (datetime.date(2019, 5, 15), DateField),
+            (datetime.datetime(2019, 5, 15), DateTimeField),
+            (datetime.time(3, 16), TimeField),
+            (datetime.timedelta(1), DurationField),
+            (Decimal('3.14'), DecimalField),
+            (b'', BinaryField),
+            (uuid.uuid4(), UUIDField),
+        ]
+        for value, output_field_type in value_types:
+            with self.subTest(type=type(value)):
+                expr = Value(value)
+                self.assertIsInstance(expr.output_field, output_field_type)
+
+    def test_resolve_output_field_failure(self):
+        msg = 'Cannot resolve expression type, unknown output_field'
+        with self.assertRaisesMessage(FieldError, msg):
+            Value(object()).output_field
+
+
+class ExistsTests(TestCase):
+    def test_optimizations(self):
+        with CaptureQueriesContext(connection) as context:
+            list(Experiment.objects.values(exists=Exists(
+                Experiment.objects.order_by('pk'),
+            )).order_by())
+        captured_queries = context.captured_queries
+        self.assertEqual(len(captured_queries), 1)
+        captured_sql = captured_queries[0]['sql']
+        self.assertNotIn(
+            connection.ops.quote_name(Experiment._meta.pk.column),
+            captured_sql,
+        )
+        self.assertIn(
+            connection.ops.limit_offset_sql(None, 1),
+            captured_sql,
+        )
+        self.assertNotIn('ORDER BY', captured_sql)
+
 
 class FieldTransformTests(TestCase):
 
@@ -1722,15 +1882,15 @@ class FieldTransformTests(TestCase):
         )
 
     def test_transform_in_values(self):
-        self.assertQuerysetEqual(
+        self.assertSequenceEqual(
             Experiment.objects.values('assigned__month'),
-            ["{'assigned__month': 6}"]
+            [{'assigned__month': 6}],
         )
 
     def test_multiple_transforms_in_values(self):
-        self.assertQuerysetEqual(
+        self.assertSequenceEqual(
             Experiment.objects.values('end__date__month'),
-            ["{'end__date__month': 6}"]
+            [{'end__date__month': 6}],
         )
 
 
@@ -1743,7 +1903,7 @@ class ReprTests(SimpleTestCase):
         )
         self.assertEqual(
             repr(When(Q(age__gte=18), then=Value('legal'))),
-            "<When: WHEN <Q: (AND: ('age__gte', 18))> THEN Value(legal)>"
+            "<When: WHEN <Q: (AND: ('age__gte', 18))> THEN Value('legal')>"
         )
         self.assertEqual(repr(Col('alias', 'field')), "Col(alias, field)")
         self.assertEqual(repr(F('published')), "F(published)")
@@ -1754,7 +1914,6 @@ class ReprTests(SimpleTestCase):
         )
         self.assertEqual(repr(Func('published', function='TO_CHAR')), "Func(F(published), function=TO_CHAR)")
         self.assertEqual(repr(OrderBy(Value(1))), 'OrderBy(Value(1), descending=False)')
-        self.assertEqual(repr(Random()), "Random()")
         self.assertEqual(repr(RawSQL('table.col', [])), "RawSQL(table.col, [])")
         self.assertEqual(repr(Ref('sum_cost', Sum('cost'))), "Ref(sum_cost, Sum(F(cost)))")
         self.assertEqual(repr(Value(1)), "Value(1)")
@@ -1830,13 +1989,59 @@ class CombinableTests(SimpleTestCase):
             object() | Combinable()
 
 
+class CombinedExpressionTests(SimpleTestCase):
+    def test_resolve_output_field(self):
+        tests = [
+            (IntegerField, AutoField, IntegerField),
+            (AutoField, IntegerField, IntegerField),
+            (IntegerField, DecimalField, DecimalField),
+            (DecimalField, IntegerField, DecimalField),
+            (IntegerField, FloatField, FloatField),
+            (FloatField, IntegerField, FloatField),
+        ]
+        connectors = [Combinable.ADD, Combinable.SUB, Combinable.MUL, Combinable.DIV]
+        for lhs, rhs, combined in tests:
+            for connector in connectors:
+                with self.subTest(lhs=lhs, connector=connector, rhs=rhs, combined=combined):
+                    expr = CombinedExpression(
+                        Expression(lhs()),
+                        connector,
+                        Expression(rhs()),
+                    )
+                    self.assertIsInstance(expr.output_field, combined)
+
+
 class ExpressionWrapperTests(SimpleTestCase):
     def test_empty_group_by(self):
         expr = ExpressionWrapper(Value(3), output_field=IntegerField())
         self.assertEqual(expr.get_group_by_cols(alias=None), [])
 
     def test_non_empty_group_by(self):
-        expr = ExpressionWrapper(Lower(Value('f')), output_field=IntegerField())
+        value = Value('f')
+        value.output_field = None
+        expr = ExpressionWrapper(Lower(value), output_field=IntegerField())
         group_by_cols = expr.get_group_by_cols(alias=None)
         self.assertEqual(group_by_cols, [expr.expression])
         self.assertEqual(group_by_cols[0].output_field, expr.output_field)
+
+
+class OrderByTests(SimpleTestCase):
+    def test_equal(self):
+        self.assertEqual(
+            OrderBy(F('field'), nulls_last=True),
+            OrderBy(F('field'), nulls_last=True),
+        )
+        self.assertNotEqual(
+            OrderBy(F('field'), nulls_last=True),
+            OrderBy(F('field'), nulls_last=False),
+        )
+
+    def test_hash(self):
+        self.assertEqual(
+            hash(OrderBy(F('field'), nulls_last=True)),
+            hash(OrderBy(F('field'), nulls_last=True)),
+        )
+        self.assertNotEqual(
+            hash(OrderBy(F('field'), nulls_last=True)),
+            hash(OrderBy(F('field'), nulls_last=False)),
+        )
