@@ -220,7 +220,7 @@ class SelectTest(WidgetTest):
             </select>"""
         ))
 
-    @override_settings(USE_L10N=True, USE_THOUSAND_SEPARATOR=True)
+    @override_settings(USE_THOUSAND_SEPARATOR=True)
     def test_doesnt_localize_option_value(self):
         choices = [
             (1, 'One'),
@@ -257,13 +257,13 @@ class SelectTest(WidgetTest):
         self.assertEqual(options[0]['value'], 'J')
         self.assertEqual(options[0]['label'], 'John')
         self.assertEqual(options[0]['index'], '0')
-        self.assertEqual(options[0]['selected'], True)
+        self.assertIs(options[0]['selected'], True)
         # Template-related attributes
         self.assertEqual(options[1]['name'], 'name')
         self.assertEqual(options[1]['value'], 'P')
         self.assertEqual(options[1]['label'], 'Paul')
         self.assertEqual(options[1]['index'], '1')
-        self.assertEqual(options[1]['selected'], False)
+        self.assertIs(options[1]['selected'], False)
 
     def test_optgroups(self):
         choices = [
@@ -336,7 +336,7 @@ class SelectTest(WidgetTest):
         )
         self.assertEqual(index, 1)
         label, options, index = unknown
-        self.assertEqual(label, None)
+        self.assertIsNone(label)
         self.assertEqual(
             options,
             [{
