@@ -89,7 +89,8 @@ class ViewTest(SimpleTestCase):
 
     def test_pathological_http_method(self):
         """
-        The edge case of a http request that spoofs an existing method name is caught.
+        The edge case of an HTTP request that spoofs an existing method name is
+        caught.
         """
         self.assertEqual(SimpleView.as_view()(
             self.rf.get('/', REQUEST_METHOD='DISPATCH')
@@ -113,6 +114,13 @@ class ViewTest(SimpleTestCase):
         response = SimpleView.as_view()(self.rf.head('/'))
         self.assertEqual(response.status_code, 200)
 
+    def test_setup_get_and_head(self):
+        view_instance = SimpleView()
+        self.assertFalse(hasattr(view_instance, 'head'))
+        view_instance.setup(self.rf.get('/'))
+        self.assertTrue(hasattr(view_instance, 'head'))
+        self.assertEqual(view_instance.head, view_instance.get)
+
     def test_head_no_get(self):
         """
         Test a view which supplies no GET method responds to HEAD with HTTP 405.
@@ -133,11 +141,11 @@ class ViewTest(SimpleTestCase):
     def test_invalid_keyword_argument(self):
         """
         View arguments must be predefined on the class and can't
-        be named like a HTTP method.
+        be named like an HTTP method.
         """
         msg = (
-            "You tried to pass in the %s method name as a keyword argument "
-            "to SimpleView(). Don't do that."
+            'The method name %s is not accepted as a keyword argument to '
+            'SimpleView().'
         )
         # Check each of the allowed method names
         for method in SimpleView.http_method_names:
@@ -165,12 +173,16 @@ class ViewTest(SimpleTestCase):
 
     def test_class_attributes(self):
         """
-        The callable returned from as_view() has proper
-        docstring, name and module.
+        The callable returned from as_view() has proper special attributes.
         """
-        self.assertEqual(SimpleView.__doc__, SimpleView.as_view().__doc__)
-        self.assertEqual(SimpleView.__name__, SimpleView.as_view().__name__)
-        self.assertEqual(SimpleView.__module__, SimpleView.as_view().__module__)
+        cls = SimpleView
+        view = cls.as_view()
+        self.assertEqual(view.__doc__, cls.__doc__)
+        self.assertEqual(view.__name__, 'view')
+        self.assertEqual(view.__module__, cls.__module__)
+        self.assertEqual(view.__qualname__, f'{cls.as_view.__qualname__}.<locals>.view')
+        self.assertEqual(view.__annotations__, cls.dispatch.__annotations__)
+        self.assertFalse(hasattr(view, '__wrapped__'))
 
     def test_dispatch_decoration(self):
         """
@@ -188,7 +200,7 @@ class ViewTest(SimpleTestCase):
         view = SimpleView.as_view()
         response = view(request)
         self.assertEqual(200, response.status_code)
-        self.assertTrue(response['Allow'])
+        self.assertTrue(response.headers['Allow'])
 
     def test_options_for_get_view(self):
         """
@@ -219,7 +231,7 @@ class ViewTest(SimpleTestCase):
 
     def _assert_allows(self, response, *expected_methods):
         "Assert allowed HTTP methods reported in the Allow response header"
-        response_allows = set(response['Allow'].split(', '))
+        response_allows = set(response.headers['Allow'].split(', '))
         self.assertEqual(set(expected_methods + ('OPTIONS',)), response_allows)
 
     def test_args_kwargs_request_on_self(self):
@@ -259,6 +271,17 @@ class ViewTest(SimpleTestCase):
         with self.assertRaisesMessage(AttributeError, msg):
             TestView.as_view()(self.rf.get('/'))
 
+    def test_setup_adds_args_kwargs_request(self):
+        request = self.rf.get('/')
+        args = ('arg 1', 'arg 2')
+        kwargs = {'kwarg_1': 1, 'kwarg_2': 'year'}
+
+        view = View()
+        view.setup(request, *args, **kwargs)
+        self.assertEqual(request, view.request)
+        self.assertEqual(args, view.args)
+        self.assertEqual(kwargs, view.kwargs)
+
     def test_direct_instantiation(self):
         """
         It should be possible to use the view by directly instantiating it
@@ -372,7 +395,7 @@ class TemplateViewTest(SimpleTestCase):
 
     def test_content_type(self):
         response = self.client.get('/template/content_type/')
-        self.assertEqual(response['Content-Type'], 'text/plain')
+        self.assertEqual(response.headers['Content-Type'], 'text/plain')
 
     def test_resolve_view(self):
         match = resolve('/template/content_type/')
@@ -443,12 +466,12 @@ class RedirectViewTest(SimpleTestCase):
         "Named pattern parameter should reverse to the matching pattern"
         response = RedirectView.as_view(pattern_name='artist_detail')(self.rf.get('/foo/'), pk=1)
         self.assertEqual(response.status_code, 302)
-        self.assertEqual(response['Location'], '/detail/artist/1/')
+        self.assertEqual(response.headers['Location'], '/detail/artist/1/')
 
     def test_named_url_pattern_using_args(self):
         response = RedirectView.as_view(pattern_name='artist_detail')(self.rf.get('/foo/'), 1)
         self.assertEqual(response.status_code, 302)
-        self.assertEqual(response['Location'], '/detail/artist/1/')
+        self.assertEqual(response.headers['Location'], '/detail/artist/1/')
 
     def test_redirect_POST(self):
         "Default is a temporary redirect"
