@@ -2,8 +2,10 @@ import os
 import shutil
 import struct
 import tempfile
+import zipfile
+from unittest import mock
 
-from django.contrib.gis.gdal import GDAL_VERSION, GDALRaster
+from django.contrib.gis.gdal import GDALRaster, SpatialReference
 from django.contrib.gis.gdal.error import GDALException
 from django.contrib.gis.gdal.raster.band import GDALBand
 from django.contrib.gis.shortcuts import numpy
@@ -146,7 +148,11 @@ class GDALRasterTests(SimpleTestCase):
 
         # Reload newly created raster from file
         restored_raster = GDALRaster(rstfile.name)
-        self.assertEqual(restored_raster.srs.wkt, self.rs.srs.wkt)
+        # Presence of TOWGS84 depend on GDAL/Proj versions.
+        self.assertEqual(
+            restored_raster.srs.wkt.replace('TOWGS84[0,0,0,0,0,0,0],', ''),
+            self.rs.srs.wkt.replace('TOWGS84[0,0,0,0,0,0,0],', '')
+        )
         self.assertEqual(restored_raster.geotransform, self.rs.geotransform)
         if numpy:
             numpy.testing.assert_equal(
@@ -156,6 +162,11 @@ class GDALRasterTests(SimpleTestCase):
         else:
             self.assertEqual(restored_raster.bands[0].data(), self.rs.bands[0].data())
 
+    def test_nonexistent_file(self):
+        msg = 'Unable to read raster source input "nonexistent.tif".'
+        with self.assertRaisesMessage(GDALException, msg):
+            GDALRaster('nonexistent.tif')
+
     def test_vsi_raster_creation(self):
         # Open a raster as a file object.
         with open(self.rs_path, 'rb') as dat:
@@ -219,6 +230,17 @@ class GDALRasterTests(SimpleTestCase):
         # The vsi buffer is None for rasters that are not vsi based.
         self.assertIsNone(self.rs.vsi_buffer)
 
+    def test_vsi_vsizip_filesystem(self):
+        rst_zipfile = tempfile.NamedTemporaryFile(suffix='.zip')
+        with zipfile.ZipFile(rst_zipfile, mode='w') as zf:
+            zf.write(self.rs_path, 'raster.tif')
+        rst_path = '/vsizip/' + os.path.join(rst_zipfile.name, 'raster.tif')
+        rst = GDALRaster(rst_path)
+        self.assertEqual(rst.driver.name, self.rs.driver.name)
+        self.assertEqual(rst.name, rst_path)
+        self.assertIs(rst.is_vsi_based, True)
+        self.assertIsNone(rst.vsi_buffer)
+
     def test_offset_size_and_shape_on_raster_creation(self):
         rast = GDALRaster({
             'datatype': 1,
@@ -260,8 +282,6 @@ class GDALRasterTests(SimpleTestCase):
         self.assertEqual(result, [23] * 4)
 
     def test_set_nodata_none_on_raster_creation(self):
-        if GDAL_VERSION < (2, 1):
-            self.skipTest("GDAL >= 2.1 is required for this test.")
         # Create raster without data and without nodata value.
         rast = GDALRaster({
             'datatype': 1,
@@ -312,59 +332,33 @@ class GDALRasterTests(SimpleTestCase):
         self.assertNotIn('OWNER', source.metadata['DEFAULT'])
 
     def test_raster_info_accessor(self):
-        if GDAL_VERSION < (2, 1):
-            msg = 'GDAL ≥ 2.1 is required for using the info property.'
-            with self.assertRaisesMessage(ValueError, msg):
-                self.rs.info
-            return
-        gdalinfo = """
-        Driver: GTiff/GeoTIFF
-        Files: {0}
-        Size is 163, 174
-        Coordinate System is:
-        PROJCS["NAD83 / Florida GDL Albers",
-            GEOGCS["NAD83",
-                DATUM["North_American_Datum_1983",
-                    SPHEROID["GRS 1980",6378137,298.257222101,
-                        AUTHORITY["EPSG","7019"]],
-                    TOWGS84[0,0,0,0,0,0,0],
-                    AUTHORITY["EPSG","6269"]],
-                PRIMEM["Greenwich",0,
-                    AUTHORITY["EPSG","8901"]],
-                UNIT["degree",0.0174532925199433,
-                    AUTHORITY["EPSG","9122"]],
-                AUTHORITY["EPSG","4269"]],
-            PROJECTION["Albers_Conic_Equal_Area"],
-            PARAMETER["standard_parallel_1",24],
-            PARAMETER["standard_parallel_2",31.5],
-            PARAMETER["latitude_of_center",24],
-            PARAMETER["longitude_of_center",-84],
-            PARAMETER["false_easting",400000],
-            PARAMETER["false_northing",0],
-            UNIT["metre",1,
-                AUTHORITY["EPSG","9001"]],
-            AXIS["X",EAST],
-            AXIS["Y",NORTH],
-            AUTHORITY["EPSG","3086"]]
-        Origin = (511700.468070655711927,435103.377123198588379)
-        Pixel Size = (100.000000000000000,-100.000000000000000)
-        Metadata:
-          AREA_OR_POINT=Area
-        Image Structure Metadata:
-          INTERLEAVE=BAND
-        Corner Coordinates:
-        Upper Left  (  511700.468,  435103.377) ( 82d51'46.16"W, 27d55' 1.53"N)
-        Lower Left  (  511700.468,  417703.377) ( 82d51'52.04"W, 27d45'37.50"N)
-        Upper Right (  528000.468,  435103.377) ( 82d41'48.81"W, 27d54'56.30"N)
-        Lower Right (  528000.468,  417703.377) ( 82d41'55.54"W, 27d45'32.28"N)
-        Center      (  519850.468,  426403.377) ( 82d46'50.64"W, 27d50'16.99"N)
-        Band 1 Block=163x50 Type=Byte, ColorInterp=Gray
-          NoData Value=15
-        """.format(self.rs_path)
+        infos = self.rs.info
         # Data
-        info_dyn = [line.strip() for line in self.rs.info.split('\n') if line.strip() != '']
-        info_ref = [line.strip() for line in gdalinfo.split('\n') if line.strip() != '']
-        self.assertEqual(info_dyn, info_ref)
+        info_lines = [line.strip() for line in infos.split('\n') if line.strip() != '']
+        for line in [
+            'Driver: GTiff/GeoTIFF',
+            'Files: {}'.format(self.rs_path),
+            'Size is 163, 174',
+            'Origin = (511700.468070655711927,435103.377123198588379)',
+            'Pixel Size = (100.000000000000000,-100.000000000000000)',
+            'Metadata:',
+            'AREA_OR_POINT=Area',
+            'Image Structure Metadata:',
+            'INTERLEAVE=BAND',
+            'Band 1 Block=163x50 Type=Byte, ColorInterp=Gray',
+            'NoData Value=15'
+        ]:
+            self.assertIn(line, info_lines)
+        for line in [
+            r'Upper Left  \(  511700.468,  435103.377\) \( 82d51\'46.1\d"W, 27d55\' 1.5\d"N\)',
+            r'Lower Left  \(  511700.468,  417703.377\) \( 82d51\'52.0\d"W, 27d45\'37.5\d"N\)',
+            r'Upper Right \(  528000.468,  435103.377\) \( 82d41\'48.8\d"W, 27d54\'56.3\d"N\)',
+            r'Lower Right \(  528000.468,  417703.377\) \( 82d41\'55.5\d"W, 27d45\'32.2\d"N\)',
+            r'Center      \(  519850.468,  426403.377\) \( 82d46\'50.6\d"W, 27d50\'16.9\d"N\)',
+        ]:
+            self.assertRegex(infos, line)
+        # CRS (skip the name because string depends on the GDAL/Proj versions).
+        self.assertIn("NAD83 / Florida GDL Albers", infos)
 
     def test_compressed_file_based_raster_creation(self):
         rstfile = tempfile.NamedTemporaryFile(suffix='.tif')
@@ -400,8 +394,7 @@ class GDALRasterTests(SimpleTestCase):
         compressed = GDALRaster(compressed.name)
         self.assertEqual(compressed.metadata['IMAGE_STRUCTURE']['COMPRESSION'], 'PACKBITS',)
         self.assertEqual(compressed.bands[0].metadata['IMAGE_STRUCTURE']['PIXELTYPE'], 'SIGNEDBYTE')
-        if GDAL_VERSION >= (2, 1):
-            self.assertIn('Block=40x23', compressed.info)
+        self.assertIn('Block=40x23', compressed.info)
 
     def test_raster_warp(self):
         # Create in memory raster
@@ -482,12 +475,129 @@ class GDALRasterTests(SimpleTestCase):
         # The result is an empty raster filled with the correct nodata value.
         self.assertEqual(result, [23] * 16)
 
-    def test_raster_transform(self):
-        # Prepare tempfile and nodata value
+    def test_raster_clone(self):
         rstfile = tempfile.NamedTemporaryFile(suffix='.tif')
-        ndv = 99
+        tests = [
+            ('MEM', '', 23),  # In memory raster.
+            ('tif', rstfile.name, 99),  # In file based raster.
+        ]
+        for driver, name, nodata_value in tests:
+            with self.subTest(driver=driver):
+                source = GDALRaster({
+                    'datatype': 1,
+                    'driver': driver,
+                    'name': name,
+                    'width': 4,
+                    'height': 4,
+                    'srid': 3086,
+                    'origin': (500000, 400000),
+                    'scale': (100, -100),
+                    'skew': (0, 0),
+                    'bands': [{
+                        'data': range(16),
+                        'nodata_value': nodata_value,
+                    }],
+                })
+                clone = source.clone()
+                self.assertNotEqual(clone.name, source.name)
+                self.assertEqual(clone._write, source._write)
+                self.assertEqual(clone.srs.srid, source.srs.srid)
+                self.assertEqual(clone.width, source.width)
+                self.assertEqual(clone.height, source.height)
+                self.assertEqual(clone.origin, source.origin)
+                self.assertEqual(clone.scale, source.scale)
+                self.assertEqual(clone.skew, source.skew)
+                self.assertIsNot(clone, source)
 
-        # Create in file based raster
+    def test_raster_transform(self):
+        tests = [
+            3086,
+            '3086',
+            SpatialReference(3086),
+        ]
+        for srs in tests:
+            with self.subTest(srs=srs):
+                # Prepare tempfile and nodata value.
+                rstfile = tempfile.NamedTemporaryFile(suffix='.tif')
+                ndv = 99
+                # Create in file based raster.
+                source = GDALRaster({
+                    'datatype': 1,
+                    'driver': 'tif',
+                    'name': rstfile.name,
+                    'width': 5,
+                    'height': 5,
+                    'nr_of_bands': 1,
+                    'srid': 4326,
+                    'origin': (-5, 5),
+                    'scale': (2, -2),
+                    'skew': (0, 0),
+                    'bands': [{
+                        'data': range(25),
+                        'nodata_value': ndv,
+                    }],
+                })
+
+                target = source.transform(srs)
+
+                # Reload data from disk.
+                target = GDALRaster(target.name)
+                self.assertEqual(target.srs.srid, 3086)
+                self.assertEqual(target.width, 7)
+                self.assertEqual(target.height, 7)
+                self.assertEqual(target.bands[0].datatype(), source.bands[0].datatype())
+                self.assertAlmostEqual(target.origin[0], 9124842.791079799, 3)
+                self.assertAlmostEqual(target.origin[1], 1589911.6476407414, 3)
+                self.assertAlmostEqual(target.scale[0], 223824.82664250192, 3)
+                self.assertAlmostEqual(target.scale[1], -223824.82664250192, 3)
+                self.assertEqual(target.skew, [0, 0])
+
+                result = target.bands[0].data()
+                if numpy:
+                    result = result.flatten().tolist()
+                # The reprojection of a raster that spans over a large area
+                # skews the data matrix and might introduce nodata values.
+                self.assertEqual(
+                    result,
+                    [
+                        ndv, ndv, ndv, ndv, 4, ndv, ndv,
+                        ndv, ndv, 2, 3, 9, ndv, ndv,
+                        ndv, 1, 2, 8, 13, 19, ndv,
+                        0, 6, 6, 12, 18, 18, 24,
+                        ndv, 10, 11, 16, 22, 23, ndv,
+                        ndv, ndv, 15, 21, 22, ndv, ndv,
+                        ndv, ndv, 20, ndv, ndv, ndv, ndv,
+                    ],
+                )
+
+    def test_raster_transform_clone(self):
+        with mock.patch.object(GDALRaster, 'clone') as mocked_clone:
+            # Create in file based raster.
+            rstfile = tempfile.NamedTemporaryFile(suffix='.tif')
+            source = GDALRaster({
+                'datatype': 1,
+                'driver': 'tif',
+                'name': rstfile.name,
+                'width': 5,
+                'height': 5,
+                'nr_of_bands': 1,
+                'srid': 4326,
+                'origin': (-5, 5),
+                'scale': (2, -2),
+                'skew': (0, 0),
+                'bands': [{
+                    'data': range(25),
+                    'nodata_value': 99,
+                }],
+            })
+            # transform() returns a clone because it is the same SRID and
+            # driver.
+            source.transform(4326)
+            self.assertEqual(mocked_clone.call_count, 1)
+
+    def test_raster_transform_clone_name(self):
+        # Create in file based raster.
+        rstfile = tempfile.NamedTemporaryFile(suffix='.tif')
         source = GDALRaster({
             'datatype': 1,
             'driver': 'tif',
@@ -501,44 +611,12 @@ class GDALRasterTests(SimpleTestCase):
             'skew': (0, 0),
             'bands': [{
                 'data': range(25),
-                'nodata_value': ndv,
+                'nodata_value': 99,
             }],
         })
-
-        # Transform raster into srid 4326.
-        target = source.transform(3086)
-
-        # Reload data from disk
-        target = GDALRaster(target.name)
-
-        self.assertEqual(target.srs.srid, 3086)
-        self.assertEqual(target.width, 7)
-        self.assertEqual(target.height, 7)
-        self.assertEqual(target.bands[0].datatype(), source.bands[0].datatype())
-        self.assertAlmostEqual(target.origin[0], 9124842.791079799, 3)
-        self.assertAlmostEqual(target.origin[1], 1589911.6476407414, 3)
-        self.assertAlmostEqual(target.scale[0], 223824.82664250192, 3)
-        self.assertAlmostEqual(target.scale[1], -223824.82664250192, 3)
-        self.assertEqual(target.skew, [0, 0])
-
-        result = target.bands[0].data()
-        if numpy:
-            result = result.flatten().tolist()
-
-        # The reprojection of a raster that spans over a large area
-        # skews the data matrix and might introduce nodata values.
-        self.assertEqual(
-            result,
-            [
-                ndv, ndv, ndv, ndv, 4, ndv, ndv,
-                ndv, ndv, 2, 3, 9, ndv, ndv,
-                ndv, 1, 2, 8, 13, 19, ndv,
-                0, 6, 6, 12, 18, 18, 24,
-                ndv, 10, 11, 16, 22, 23, ndv,
-                ndv, ndv, 15, 21, 22, ndv, ndv,
-                ndv, ndv, 20, ndv, ndv, ndv, ndv,
-            ]
-        )
+        clone_name = rstfile.name + '_respect_name.GTiff'
+        target = source.transform(4326, name=clone_name)
+        self.assertEqual(target.name, clone_name)
 
 
 class GDALBandTests(SimpleTestCase):
@@ -722,13 +800,8 @@ class GDALBandTests(SimpleTestCase):
             'height': 1,
             'bands': [{'data': [0], 'nodata_value': 1}],
         })
-        if GDAL_VERSION < (2, 1):
-            msg = 'GDAL >= 2.1 required to delete nodata values.'
-            with self.assertRaisesMessage(ValueError, msg):
-                rsmem.bands[0].nodata_value = None
-        else:
-            rsmem.bands[0].nodata_value = None
-            self.assertIsNone(rsmem.bands[0].nodata_value)
+        rsmem.bands[0].nodata_value = None
+        self.assertIsNone(rsmem.bands[0].nodata_value)
 
     def test_band_data_replication(self):
         band = GDALRaster({
