@@ -1,18 +1,14 @@
-# -*- coding: utf-8 -*-
-
-from __future__ import unicode_literals
-
+from django.core.exceptions import ImproperlyConfigured
 from django.core.handlers.wsgi import WSGIHandler, WSGIRequest, get_script_name
 from django.core.signals import request_finished, request_started
 from django.db import close_old_connections, connection
 from django.test import (
     RequestFactory, SimpleTestCase, TransactionTestCase, override_settings,
 )
-from django.utils import six
-from django.utils.encoding import force_str
 
 
 class HandlerTests(SimpleTestCase):
+    request_factory = RequestFactory()
 
     def setUp(self):
         request_started.disconnect(close_old_connections)
@@ -20,33 +16,27 @@ class HandlerTests(SimpleTestCase):
     def tearDown(self):
         request_started.connect(close_old_connections)
 
-    # Mangle settings so the handler will fail
-    @override_settings(MIDDLEWARE_CLASSES=42)
-    def test_lock_safety(self):
-        """
-        Tests for bug #11193 (errors inside middleware shouldn't leave
-        the initLock locked).
-        """
-        # Try running the handler, it will fail in load_middleware
+    def test_middleware_initialized(self):
         handler = WSGIHandler()
-        self.assertEqual(handler.initLock.locked(), False)
-        with self.assertRaises(Exception):
-            handler(None, None)
-        self.assertEqual(handler.initLock.locked(), False)
+        self.assertIsNotNone(handler._middleware_chain)
 
     def test_bad_path_info(self):
-        """Tests for bug #15672 ('request' referenced before assignment)"""
-        environ = RequestFactory().get('/').environ
-        environ['PATH_INFO'] = b'\xed' if six.PY2 else '\xed'
+        """
+        A non-UTF-8 path populates PATH_INFO with an URL-encoded path and
+        produces a 404.
+        """
+        environ = self.request_factory.get('/').environ
+        environ['PATH_INFO'] = '\xed'
         handler = WSGIHandler()
         response = handler(environ, lambda *a, **k: None)
-        self.assertEqual(response.status_code, 400)
+        # The path of the request will be encoded to '/%ED'.
+        self.assertEqual(response.status_code, 404)
 
     def test_non_ascii_query_string(self):
         """
-        Test that non-ASCII query strings are properly decoded (#20530, #22996).
+        Non-ASCII query strings are properly decoded (#20530, #22996).
         """
-        environ = RequestFactory().get('/').environ
+        environ = self.request_factory.get('/').environ
         raw_query_strings = [
             b'want=caf%C3%A9',  # This is the proper way to encode 'café'
             b'want=caf\xc3\xa9',  # UA forgot to quote bytes
@@ -55,38 +45,27 @@ class HandlerTests(SimpleTestCase):
         ]
         got = []
         for raw_query_string in raw_query_strings:
-            if six.PY3:
-                # Simulate http.server.BaseHTTPRequestHandler.parse_request handling of raw request
-                environ['QUERY_STRING'] = str(raw_query_string, 'iso-8859-1')
-            else:
-                environ['QUERY_STRING'] = raw_query_string
+            # Simulate http.server.BaseHTTPRequestHandler.parse_request handling of raw request
+            environ['QUERY_STRING'] = str(raw_query_string, 'iso-8859-1')
             request = WSGIRequest(environ)
             got.append(request.GET['want'])
-        if six.PY2:
-            self.assertListEqual(got, ['café', 'café', 'café', 'café'])
-        else:
-            # On Python 3, %E9 is converted to the unicode replacement character by parse_qsl
-            self.assertListEqual(got, ['café', 'café', 'caf\ufffd', 'café'])
+        # %E9 is converted to the Unicode replacement character by parse_qsl
+        self.assertEqual(got, ['café', 'café', 'caf\ufffd', 'café'])
 
     def test_non_ascii_cookie(self):
-        """Test that non-ASCII cookies set in JavaScript are properly decoded (#20557)."""
-        environ = RequestFactory().get('/').environ
-        raw_cookie = 'want="café"'
-        if six.PY3:
-            raw_cookie = raw_cookie.encode('utf-8').decode('iso-8859-1')
+        """Non-ASCII cookies set in JavaScript are properly decoded (#20557)."""
+        environ = self.request_factory.get('/').environ
+        raw_cookie = 'want="café"'.encode('utf-8').decode('iso-8859-1')
         environ['HTTP_COOKIE'] = raw_cookie
         request = WSGIRequest(environ)
-        # If would be nicer if request.COOKIES returned unicode values.
-        # However the current cookie parser doesn't do this and fixing it is
-        # much more work than fixing #20557. Feel free to remove force_str()!
-        self.assertEqual(request.COOKIES['want'], force_str("café"))
+        self.assertEqual(request.COOKIES['want'], "café")
 
     def test_invalid_unicode_cookie(self):
         """
         Invalid cookie content should result in an absent cookie, but not in a
         crash while trying to decode it (#23638).
         """
-        environ = RequestFactory().get('/').environ
+        environ = self.request_factory.get('/').environ
         environ['HTTP_COOKIE'] = 'x=W\x03c(h]\x8e'
         request = WSGIRequest(environ)
         # We don't test COOKIES content, as the result might differ between
@@ -100,7 +79,7 @@ class HandlerTests(SimpleTestCase):
         Invalid boundary string should produce a "Bad Request" response, not a
         server error (#23887).
         """
-        environ = RequestFactory().post('/malformed_post/').environ
+        environ = self.request_factory.post('/malformed_post/').environ
         environ['CONTENT_TYPE'] = 'multipart/form-data; boundary=WRONG\x07'
         handler = WSGIHandler()
         response = handler(environ, lambda *a, **k: None)
@@ -108,7 +87,7 @@ class HandlerTests(SimpleTestCase):
         self.assertEqual(response.status_code, 400)
 
 
-@override_settings(ROOT_URLCONF='handlers.urls')
+@override_settings(ROOT_URLCONF='handlers.urls', MIDDLEWARE=[])
 class TransactionsPerRequestTests(TransactionTestCase):
 
     available_apps = []
@@ -126,6 +105,16 @@ class TransactionsPerRequestTests(TransactionTestCase):
             connection.settings_dict['ATOMIC_REQUESTS'] = old_atomic_requests
         self.assertContains(response, 'True')
 
+    async def test_auto_transaction_async_view(self):
+        old_atomic_requests = connection.settings_dict['ATOMIC_REQUESTS']
+        try:
+            connection.settings_dict['ATOMIC_REQUESTS'] = True
+            msg = 'You cannot use ATOMIC_REQUESTS with async views.'
+            with self.assertRaisesMessage(RuntimeError, msg):
+                await self.async_client.get('/async_regular/')
+        finally:
+            connection.settings_dict['ATOMIC_REQUESTS'] = old_atomic_requests
+
     def test_no_auto_transaction(self):
         old_atomic_requests = connection.settings_dict['ATOMIC_REQUESTS']
         try:
@@ -169,37 +158,68 @@ class SignalsTests(SimpleTestCase):
         self.assertEqual(self.signals, ['started', 'finished'])
 
 
+def empty_middleware(get_response):
+    pass
+
+
 @override_settings(ROOT_URLCONF='handlers.urls')
-class HandlerSuspiciousOpsTest(SimpleTestCase):
+class HandlerRequestTests(SimpleTestCase):
+    request_factory = RequestFactory()
+
+    def test_async_view(self):
+        """Calling an async view down the normal synchronous path."""
+        response = self.client.get('/async_regular/')
+        self.assertEqual(response.status_code, 200)
 
     def test_suspiciousop_in_view_returns_400(self):
         response = self.client.get('/suspicious/')
         self.assertEqual(response.status_code, 400)
 
-
-@override_settings(ROOT_URLCONF='handlers.urls')
-class HandlerNotFoundTest(SimpleTestCase):
+    def test_bad_request_in_view_returns_400(self):
+        response = self.client.get('/bad_request/')
+        self.assertEqual(response.status_code, 400)
 
     def test_invalid_urls(self):
         response = self.client.get('~%A9helloworld')
         self.assertEqual(response.status_code, 404)
-        self.assertContains(response, '~%A9helloworld', status_code=404)
+        self.assertEqual(response.context['request_path'], '/~%25A9helloworld')
 
         response = self.client.get('d%aao%aaw%aan%aal%aao%aaa%aad%aa/')
-        self.assertEqual(response.status_code, 404)
-        self.assertContains(response, 'd%AAo%AAw%AAn%AAl%AAo%AAa%AAd%AA', status_code=404)
+        self.assertEqual(response.context['request_path'], '/d%25AAo%25AAw%25AAn%25AAl%25AAo%25AAa%25AAd%25AA')
 
         response = self.client.get('/%E2%99%E2%99%A5/')
-        self.assertEqual(response.status_code, 404)
-        self.assertContains(response, '%E2%99\u2665', status_code=404)
+        self.assertEqual(response.context['request_path'], '/%25E2%2599%E2%99%A5/')
 
         response = self.client.get('/%E2%98%8E%E2%A9%E2%99%A5/')
-        self.assertEqual(response.status_code, 404)
-        self.assertContains(response, '\u260e%E2%A9\u2665', status_code=404)
+        self.assertEqual(response.context['request_path'], '/%E2%98%8E%25E2%25A9%E2%99%A5/')
 
     def test_environ_path_info_type(self):
-        environ = RequestFactory().get('/%E2%A8%87%87%A5%E2%A8%A0').environ
-        self.assertIsInstance(environ['PATH_INFO'], six.text_type)
+        environ = self.request_factory.get('/%E2%A8%87%87%A5%E2%A8%A0').environ
+        self.assertIsInstance(environ['PATH_INFO'], str)
+
+    def test_handle_accepts_httpstatus_enum_value(self):
+        def start_response(status, headers):
+            start_response.status = status
+
+        environ = self.request_factory.get('/httpstatus_enum/').environ
+        WSGIHandler()(environ, start_response)
+        self.assertEqual(start_response.status, '200 OK')
+
+    @override_settings(MIDDLEWARE=['handlers.tests.empty_middleware'])
+    def test_middleware_returns_none(self):
+        msg = 'Middleware factory handlers.tests.empty_middleware returned None.'
+        with self.assertRaisesMessage(ImproperlyConfigured, msg):
+            self.client.get('/')
+
+    def test_no_response(self):
+        msg = "The view %s didn't return an HttpResponse object. It returned None instead."
+        tests = (
+            ('/no_response_fbv/', 'handlers.views.no_response'),
+            ('/no_response_cbv/', 'handlers.views.NoResponse.__call__'),
+        )
+        for url, view in tests:
+            with self.subTest(url=url), self.assertRaisesMessage(ValueError, msg % view):
+                self.client.get(url)
 
 
 class ScriptNameTests(SimpleTestCase):
@@ -211,3 +231,55 @@ class ScriptNameTests(SimpleTestCase):
 
         script_name = get_script_name({'SCRIPT_URL': '/foobar/', 'PATH_INFO': '/'})
         self.assertEqual(script_name, '/foobar')
+
+    def test_get_script_name_double_slashes(self):
+        """
+        WSGI squashes multiple successive slashes in PATH_INFO, get_script_name
+        should take that into account when forming SCRIPT_NAME (#17133).
+        """
+        script_name = get_script_name({
+            'SCRIPT_URL': '/mst/milestones//accounts/login//help',
+            'PATH_INFO': '/milestones/accounts/login/help',
+        })
+        self.assertEqual(script_name, '/mst')
+
+
+@override_settings(ROOT_URLCONF='handlers.urls')
+class AsyncHandlerRequestTests(SimpleTestCase):
+    """Async variants of the normal handler request tests."""
+
+    async def test_sync_view(self):
+        """Calling a sync view down the asynchronous path."""
+        response = await self.async_client.get('/regular/')
+        self.assertEqual(response.status_code, 200)
+
+    async def test_async_view(self):
+        """Calling an async view down the asynchronous path."""
+        response = await self.async_client.get('/async_regular/')
+        self.assertEqual(response.status_code, 200)
+
+    async def test_suspiciousop_in_view_returns_400(self):
+        response = await self.async_client.get('/suspicious/')
+        self.assertEqual(response.status_code, 400)
+
+    async def test_bad_request_in_view_returns_400(self):
+        response = await self.async_client.get('/bad_request/')
+        self.assertEqual(response.status_code, 400)
+
+    async def test_no_response(self):
+        msg = (
+            "The view handlers.views.no_response didn't return an "
+            "HttpResponse object. It returned None instead."
+        )
+        with self.assertRaisesMessage(ValueError, msg):
+            await self.async_client.get('/no_response_fbv/')
+
+    async def test_unawaited_response(self):
+        msg = (
+            "The view handlers.views.CoroutineClearingView.__call__ didn't"
+            " return an HttpResponse object. It returned an unawaited"
+            " coroutine instead. You may need to add an 'await'"
+            " into your view."
+        )
+        with self.assertRaisesMessage(ValueError, msg):
+            await self.async_client.get('/unawaited/')
