@@ -8,7 +8,7 @@ from django.db import connection
 from django.db.backends.base.introspection import TableInfo
 from django.test import TestCase, TransactionTestCase, skipUnlessDBFeature
 
-from .models import PeopleMoreData
+from .models import PeopleMoreData, test_collation
 
 
 def inspectdb_tables_only(table_name):
@@ -19,6 +19,13 @@ def inspectdb_tables_only(table_name):
     return table_name.startswith('inspectdb_')
 
 
+def inspectdb_views_only(table_name):
+    return (
+        table_name.startswith('inspectdb_') and
+        table_name.endswith(('_materialized', '_view'))
+    )
+
+
 def special_table_only(table_name):
     return table_name.startswith('inspectdb_special')
 
@@ -61,10 +68,11 @@ class InspectDBTestCase(TestCase):
     def test_field_types(self):
         """Test introspection of various Django field types"""
         assertFieldType = self.make_field_type_asserter()
-
+        introspected_field_types = connection.features.introspected_field_types
+        char_field_type = introspected_field_types['CharField']
         # Inspecting Oracle DB doesn't produce correct results (#19884):
         # - it reports fields as blank=True when they aren't.
-        if not connection.features.interprets_empty_strings_as_nulls:
+        if not connection.features.interprets_empty_strings_as_nulls and char_field_type == 'CharField':
             assertFieldType('char_field', "models.CharField(max_length=10)")
             assertFieldType('null_char_field', "models.CharField(max_length=10, blank=True, null=True)")
             assertFieldType('email_field', "models.CharField(max_length=254)")
@@ -73,14 +81,22 @@ class InspectDBTestCase(TestCase):
             assertFieldType('slug_field', "models.CharField(max_length=50)")
             assertFieldType('text_field', "models.TextField()")
             assertFieldType('url_field', "models.CharField(max_length=200)")
+        if char_field_type == 'TextField':
+            assertFieldType('char_field', 'models.TextField()')
+            assertFieldType('null_char_field', 'models.TextField(blank=True, null=True)')
+            assertFieldType('email_field', 'models.TextField()')
+            assertFieldType('file_field', 'models.TextField()')
+            assertFieldType('file_path_field', 'models.TextField()')
+            assertFieldType('slug_field', 'models.TextField()')
+            assertFieldType('text_field', 'models.TextField()')
+            assertFieldType('url_field', 'models.TextField()')
         assertFieldType('date_field', "models.DateField()")
         assertFieldType('date_time_field', "models.DateTimeField()")
-        if connection.features.can_introspect_ip_address_field:
+        if introspected_field_types['GenericIPAddressField'] == 'GenericIPAddressField':
             assertFieldType('gen_ip_address_field', "models.GenericIPAddressField()")
         elif not connection.features.interprets_empty_strings_as_nulls:
             assertFieldType('gen_ip_address_field', "models.CharField(max_length=39)")
-        if connection.features.can_introspect_time_field:
-            assertFieldType('time_field', "models.TimeField()")
+        assertFieldType('time_field', 'models.%s()' % introspected_field_types['TimeField'])
         if connection.features.has_native_uuid_field:
             assertFieldType('uuid_field', "models.UUIDField()")
         elif not connection.features.interprets_empty_strings_as_nulls:
@@ -95,23 +111,59 @@ class InspectDBTestCase(TestCase):
             self.assertIn('json_field = models.JSONField()', output)
         self.assertIn('null_json_field = models.JSONField(blank=True, null=True)', output)
 
+    @skipUnlessDBFeature('supports_collation_on_charfield')
+    @skipUnless(test_collation, 'Language collations are not supported.')
+    def test_char_field_db_collation(self):
+        out = StringIO()
+        call_command('inspectdb', 'inspectdb_charfielddbcollation', stdout=out)
+        output = out.getvalue()
+        if not connection.features.interprets_empty_strings_as_nulls:
+            self.assertIn(
+                "char_field = models.CharField(max_length=10, "
+                "db_collation='%s')" % test_collation,
+                output,
+            )
+        else:
+            self.assertIn(
+                "char_field = models.CharField(max_length=10, "
+                "db_collation='%s', blank=True, null=True)" % test_collation,
+                output,
+            )
+
+    @skipUnlessDBFeature('supports_collation_on_textfield')
+    @skipUnless(test_collation, 'Language collations are not supported.')
+    def test_text_field_db_collation(self):
+        out = StringIO()
+        call_command('inspectdb', 'inspectdb_textfielddbcollation', stdout=out)
+        output = out.getvalue()
+        if not connection.features.interprets_empty_strings_as_nulls:
+            self.assertIn(
+                "text_field = models.TextField(db_collation='%s')" % test_collation,
+                output,
+            )
+        else:
+            self.assertIn(
+                "text_field = models.TextField(db_collation='%s, blank=True, "
+                "null=True)" % test_collation,
+                output,
+            )
+
     def test_number_field_types(self):
         """Test introspection of various Django field types"""
         assertFieldType = self.make_field_type_asserter()
+        introspected_field_types = connection.features.introspected_field_types
 
-        if not connection.features.can_introspect_autofield:
-            assertFieldType('id', "models.IntegerField(primary_key=True)  # AutoField?")
+        auto_field_type = connection.features.introspected_field_types['AutoField']
+        if auto_field_type != 'AutoField':
+            assertFieldType('id', "models.%s(primary_key=True)  # AutoField?" % auto_field_type)
 
-        if connection.features.can_introspect_big_integer_field:
-            assertFieldType('big_int_field', "models.BigIntegerField()")
-        else:
-            assertFieldType('big_int_field', "models.IntegerField()")
+        assertFieldType('big_int_field', 'models.%s()' % introspected_field_types['BigIntegerField'])
 
-        bool_field_type = connection.features.introspected_boolean_field_type
+        bool_field_type = introspected_field_types['BooleanField']
         assertFieldType('bool_field', "models.{}()".format(bool_field_type))
         assertFieldType('null_bool_field', 'models.{}(blank=True, null=True)'.format(bool_field_type))
 
-        if connection.features.can_introspect_decimal_field:
+        if connection.vendor != 'sqlite':
             assertFieldType('decimal_field', "models.DecimalField(max_digits=6, decimal_places=1)")
         else:       # Guessed arguments on SQLite, see #5014
             assertFieldType('decimal_field', "models.DecimalField(max_digits=10, decimal_places=5)  "
@@ -119,37 +171,11 @@ class InspectDBTestCase(TestCase):
                                              "as this database handles decimal fields as float")
 
         assertFieldType('float_field', "models.FloatField()")
-
-        assertFieldType('int_field', "models.IntegerField()")
-
-        if connection.features.can_introspect_positive_integer_field:
-            assertFieldType('pos_int_field', "models.PositiveIntegerField()")
-        else:
-            assertFieldType('pos_int_field', "models.IntegerField()")
-
-        if connection.features.can_introspect_positive_integer_field:
-            if connection.features.can_introspect_big_integer_field:
-                assertFieldType('pos_big_int_field', 'models.PositiveBigIntegerField()')
-            else:
-                assertFieldType('pos_big_int_field', 'models.PositiveIntegerField()')
-            if connection.features.can_introspect_small_integer_field:
-                assertFieldType('pos_small_int_field', "models.PositiveSmallIntegerField()")
-            else:
-                assertFieldType('pos_small_int_field', "models.PositiveIntegerField()")
-        else:
-            if connection.features.can_introspect_big_integer_field:
-                assertFieldType('pos_big_int_field', 'models.BigIntegerField()')
-            else:
-                assertFieldType('pos_big_int_field', 'models.IntegerField()')
-            if connection.features.can_introspect_small_integer_field:
-                assertFieldType('pos_small_int_field', "models.SmallIntegerField()")
-            else:
-                assertFieldType('pos_small_int_field', "models.IntegerField()")
-
-        if connection.features.can_introspect_small_integer_field:
-            assertFieldType('small_int_field', "models.SmallIntegerField()")
-        else:
-            assertFieldType('small_int_field', "models.IntegerField()")
+        assertFieldType('int_field', 'models.%s()' % introspected_field_types['IntegerField'])
+        assertFieldType('pos_int_field', 'models.%s()' % introspected_field_types['PositiveIntegerField'])
+        assertFieldType('pos_big_int_field', 'models.%s()' % introspected_field_types['PositiveBigIntegerField'])
+        assertFieldType('pos_small_int_field', 'models.%s()' % introspected_field_types['PositiveSmallIntegerField'])
+        assertFieldType('small_int_field', 'models.%s()' % introspected_field_types['SmallIntegerField'])
 
     @skipUnlessDBFeature('can_introspect_foreign_keys')
     def test_attribute_name_not_python_keyword(self):
@@ -180,19 +206,20 @@ class InspectDBTestCase(TestCase):
 
     def test_digits_column_name_introspection(self):
         """Introspection of column names consist/start with digits (#16536/#17676)"""
+        char_field_type = connection.features.introspected_field_types['CharField']
         out = StringIO()
         call_command('inspectdb', 'inspectdb_digitsincolumnname', stdout=out)
         output = out.getvalue()
         error_message = "inspectdb generated a model field name which is a number"
-        self.assertNotIn("    123 = models.CharField", output, msg=error_message)
-        self.assertIn("number_123 = models.CharField", output)
+        self.assertNotIn('    123 = models.%s' % char_field_type, output, msg=error_message)
+        self.assertIn('number_123 = models.%s' % char_field_type, output)
 
         error_message = "inspectdb generated a model field name which starts with a digit"
-        self.assertNotIn("    4extra = models.CharField", output, msg=error_message)
-        self.assertIn("number_4extra = models.CharField", output)
+        self.assertNotIn('    4extra = models.%s' % char_field_type, output, msg=error_message)
+        self.assertIn('number_4extra = models.%s' % char_field_type, output)
 
-        self.assertNotIn("    45extra = models.CharField", output, msg=error_message)
-        self.assertIn("number_45extra = models.CharField", output)
+        self.assertNotIn('    45extra = models.%s' % char_field_type, output, msg=error_message)
+        self.assertIn('number_45extra = models.%s' % char_field_type, output)
 
     def test_special_column_name_introspection(self):
         """
@@ -203,12 +230,13 @@ class InspectDBTestCase(TestCase):
         call_command('inspectdb', table_name_filter=special_table_only, stdout=out)
         output = out.getvalue()
         base_name = connection.introspection.identifier_converter('Field')
-        self.assertIn("field = models.IntegerField()", output)
-        self.assertIn("field_field = models.IntegerField(db_column='%s_')" % base_name, output)
-        self.assertIn("field_field_0 = models.IntegerField(db_column='%s__')" % base_name, output)
-        self.assertIn("field_field_1 = models.IntegerField(db_column='__field')", output)
-        self.assertIn("prc_x = models.IntegerField(db_column='prc(%) x')", output)
-        self.assertIn("tamaño = models.IntegerField()", output)
+        integer_field_type = connection.features.introspected_field_types['IntegerField']
+        self.assertIn("field = models.%s()" % integer_field_type, output)
+        self.assertIn("field_field = models.%s(db_column='%s_')" % (integer_field_type, base_name), output)
+        self.assertIn("field_field_0 = models.%s(db_column='%s__')" % (integer_field_type, base_name), output)
+        self.assertIn("field_field_1 = models.%s(db_column='__field')" % integer_field_type, output)
+        self.assertIn("prc_x = models.{}(db_column='prc(%) x')".format(integer_field_type), output)
+        self.assertIn("tamaño = models.%s()" % integer_field_type, output)
 
     def test_table_name_introspection(self):
         """
@@ -316,11 +344,20 @@ class InspectDBTransactionalTests(TransactionTestCase):
         view_model = 'class InspectdbPeopleView(models.Model):'
         view_managed = 'managed = False  # Created from a view.'
         try:
-            call_command('inspectdb', table_name_filter=inspectdb_tables_only, stdout=out)
+            call_command(
+                'inspectdb',
+                table_name_filter=inspectdb_views_only,
+                stdout=out,
+            )
             no_views_output = out.getvalue()
             self.assertNotIn(view_model, no_views_output)
             self.assertNotIn(view_managed, no_views_output)
-            call_command('inspectdb', table_name_filter=inspectdb_tables_only, include_views=True, stdout=out)
+            call_command(
+                'inspectdb',
+                table_name_filter=inspectdb_views_only,
+                include_views=True,
+                stdout=out,
+            )
             with_views_output = out.getvalue()
             self.assertIn(view_model, with_views_output)
             self.assertIn(view_managed, with_views_output)
@@ -340,11 +377,20 @@ class InspectDBTransactionalTests(TransactionTestCase):
         view_model = 'class InspectdbPeopleMaterialized(models.Model):'
         view_managed = 'managed = False  # Created from a view.'
         try:
-            call_command('inspectdb', table_name_filter=inspectdb_tables_only, stdout=out)
+            call_command(
+                'inspectdb',
+                table_name_filter=inspectdb_views_only,
+                stdout=out,
+            )
             no_views_output = out.getvalue()
             self.assertNotIn(view_model, no_views_output)
             self.assertNotIn(view_managed, no_views_output)
-            call_command('inspectdb', table_name_filter=inspectdb_tables_only, include_views=True, stdout=out)
+            call_command(
+                'inspectdb',
+                table_name_filter=inspectdb_views_only,
+                include_views=True,
+                stdout=out,
+            )
             with_views_output = out.getvalue()
             self.assertIn(view_model, with_views_output)
             self.assertIn(view_managed, with_views_output)
@@ -353,7 +399,6 @@ class InspectDBTransactionalTests(TransactionTestCase):
                 cursor.execute('DROP MATERIALIZED VIEW inspectdb_people_materialized')
 
     @skipUnless(connection.vendor == 'postgresql', 'PostgreSQL specific SQL')
-    @skipUnlessDBFeature('supports_table_partitions')
     def test_include_partitions(self):
         """inspectdb --include-partitions creates models for partitions."""
         with connection.cursor() as cursor:
@@ -405,7 +450,11 @@ class InspectDBTransactionalTests(TransactionTestCase):
         foreign_table_model = 'class InspectdbIrisForeignTable(models.Model):'
         foreign_table_managed = 'managed = False'
         try:
-            call_command('inspectdb', stdout=out)
+            call_command(
+                'inspectdb',
+                table_name_filter=inspectdb_tables_only,
+                stdout=out,
+            )
             output = out.getvalue()
             self.assertIn(foreign_table_model, output)
             self.assertIn(foreign_table_managed, output)
