@@ -722,6 +722,13 @@ class ModelAdminTests(TestCase):
         self.assertEqual(perms_needed, {'band'})
         self.assertEqual(protected, [])
 
+    def test_modeladmin_repr(self):
+        ma = ModelAdmin(Band, self.site)
+        self.assertEqual(
+            repr(ma),
+            "<ModelAdmin: model=Band site=AdminSite(name='admin')>",
+        )
+
 
 class ModelAdminPermissionTests(SimpleTestCase):
 
