@@ -1,10 +1,9 @@
-from django.core.serializers.json import DjangoJSONEncoder
 from django.db import models
 
 from .fields import (
     ArrayField, BigIntegerRangeField, CICharField, CIEmailField, CITextField,
     DateRangeField, DateTimeRangeField, DecimalRangeField, EnumField,
-    HStoreField, IntegerRangeField, JSONField, SearchVectorField,
+    HStoreField, IntegerRangeField, SearchVectorField,
 )
 
 
@@ -46,6 +45,8 @@ class IntegerArrayModel(PostgreSQLModel):
 
 class NullableIntegerArrayModel(PostgreSQLModel):
     field = ArrayField(models.IntegerField(), blank=True, null=True)
+    field_nested = ArrayField(ArrayField(models.IntegerField(null=True)), null=True)
+    order = models.IntegerField(null=True)
 
 
 class CharArrayModel(PostgreSQLModel):
@@ -67,7 +68,7 @@ class OtherTypesArrayModel(PostgreSQLModel):
     uuids = ArrayField(models.UUIDField(), default=list)
     decimals = ArrayField(models.DecimalField(max_digits=5, decimal_places=2), default=list)
     tags = ArrayField(TagField(), blank=True, null=True)
-    json = ArrayField(JSONField(default=dict), default=list)
+    json = ArrayField(models.JSONField(default=dict), default=list)
     int_ranges = ArrayField(IntegerRangeField(), blank=True, null=True)
     bigint_ranges = ArrayField(BigIntegerRangeField(), blank=True, null=True)
 
@@ -82,14 +83,19 @@ class ArrayEnumModel(PostgreSQLModel):
 
 
 class CharFieldModel(models.Model):
-    field = models.CharField(max_length=16)
+    field = models.CharField(max_length=64)
 
 
 class TextFieldModel(models.Model):
     field = models.TextField()
 
-    def __str__(self):
-        return self.field
+
+class SmallAutoFieldModel(models.Model):
+    id = models.SmallAutoField(primary_key=True)
+
+
+class BigAutoFieldModel(models.Model):
+    id = models.BigAutoField(primary_key=True)
 
 
 # Scene/Character/Line models are used to test full text search. They're
@@ -98,16 +104,10 @@ class Scene(models.Model):
     scene = models.CharField(max_length=255)
     setting = models.CharField(max_length=255)
 
-    def __str__(self):
-        return self.scene
-
 
 class Character(models.Model):
     name = models.CharField(max_length=255)
 
-    def __str__(self):
-        return self.name
-
 
 class CITestModel(PostgreSQLModel):
     name = CICharField(primary_key=True, max_length=255)
@@ -115,9 +115,6 @@ class CITestModel(PostgreSQLModel):
     description = CITextField()
     array_field = ArrayField(CITextField(), null=True)
 
-    def __str__(self):
-        return self.name
-
 
 class Line(PostgreSQLModel):
     scene = models.ForeignKey('Scene', models.CASCADE)
@@ -126,8 +123,10 @@ class Line(PostgreSQLModel):
     dialogue_search_vector = SearchVectorField(blank=True, null=True)
     dialogue_config = models.CharField(max_length=100, blank=True, null=True)
 
-    def __str__(self):
-        return self.dialogue or ''
+
+class LineSavedSearch(PostgreSQLModel):
+    line = models.ForeignKey('Line', models.CASCADE)
+    query = models.CharField(max_length=100)
 
 
 class RangesModel(PostgreSQLModel):
@@ -147,11 +146,8 @@ class RangeLookupsModel(PostgreSQLModel):
     float = models.FloatField(blank=True, null=True)
     timestamp = models.DateTimeField(blank=True, null=True)
     date = models.DateField(blank=True, null=True)
-
-
-class JSONModel(PostgreSQLModel):
-    field = JSONField(blank=True, null=True)
-    field_custom = JSONField(blank=True, null=True, encoder=DjangoJSONEncoder)
+    small_integer = models.SmallIntegerField(blank=True, null=True)
+    decimal_field = models.DecimalField(max_digits=5, decimal_places=2, blank=True, null=True)
 
 
 class ArrayFieldSubclass(ArrayField):
@@ -159,16 +155,17 @@ class ArrayFieldSubclass(ArrayField):
         super().__init__(models.IntegerField())
 
 
-class AggregateTestModel(models.Model):
+class AggregateTestModel(PostgreSQLModel):
     """
     To test postgres-specific general aggregation functions
     """
     char_field = models.CharField(max_length=30, blank=True)
     integer_field = models.IntegerField(null=True)
     boolean_field = models.BooleanField(null=True)
+    json_field = models.JSONField(null=True)
 
 
-class StatTestModel(models.Model):
+class StatTestModel(PostgreSQLModel):
     """
     To test postgres-specific aggregation functions for statistics
     """
@@ -195,3 +192,4 @@ class HotelReservation(PostgreSQLModel):
     start = models.DateTimeField()
     end = models.DateTimeField()
     cancelled = models.BooleanField(default=False)
+    requirements = models.JSONField(blank=True, null=True)
