@@ -9,8 +9,8 @@ from django.core import checks, exceptions, serializers, validators
 from django.core.exceptions import FieldError
 from django.core.management import call_command
 from django.db import IntegrityError, connection, models
-from django.db.models.expressions import RawSQL
-from django.db.models.functions import Cast
+from django.db.models.expressions import Exists, OuterRef, RawSQL, Value
+from django.db.models.functions import Cast, JSONObject, Upper
 from django.test import TransactionTestCase, modify_settings, override_settings
 from django.test.utils import isolate_apps
 from django.utils import timezone
@@ -25,17 +25,68 @@ from .models import (
 )
 
 try:
+    from psycopg2.extras import NumericRange
+
     from django.contrib.postgres.aggregates import ArrayAgg
+    from django.contrib.postgres.expressions import ArraySubquery
     from django.contrib.postgres.fields import ArrayField
-    from django.contrib.postgres.fields.array import IndexTransform, SliceTransform
+    from django.contrib.postgres.fields.array import (
+        IndexTransform, SliceTransform,
+    )
     from django.contrib.postgres.forms import (
         SimpleArrayField, SplitArrayField, SplitArrayWidget,
     )
-    from psycopg2.extras import NumericRange
 except ImportError:
     pass
 
 
+@isolate_apps('postgres_tests')
+class BasicTests(PostgreSQLSimpleTestCase):
+    def test_get_field_display(self):
+        class MyModel(PostgreSQLModel):
+            field = ArrayField(
+                models.CharField(max_length=16),
+                choices=[
+                    ['Media', [(['vinyl', 'cd'], 'Audio')]],
+                    (('mp3', 'mp4'), 'Digital'),
+                ],
+            )
+
+        tests = (
+            (['vinyl', 'cd'], 'Audio'),
+            (('mp3', 'mp4'), 'Digital'),
+            (('a', 'b'), "('a', 'b')"),
+            (['c', 'd'], "['c', 'd']"),
+        )
+        for value, display in tests:
+            with self.subTest(value=value, display=display):
+                instance = MyModel(field=value)
+                self.assertEqual(instance.get_field_display(), display)
+
+    def test_get_field_display_nested_array(self):
+        class MyModel(PostgreSQLModel):
+            field = ArrayField(
+                ArrayField(models.CharField(max_length=16)),
+                choices=[
+                    [
+                        'Media',
+                        [([['vinyl', 'cd'], ('x',)], 'Audio')],
+                    ],
+                    ((['mp3'], ('mp4',)), 'Digital'),
+                ],
+            )
+        tests = (
+            ([['vinyl', 'cd'], ('x',)], 'Audio'),
+            ((['mp3'], ('mp4',)), 'Digital'),
+            ((('a', 'b'), ('c',)), "(('a', 'b'), ('c',))"),
+            ([['a', 'b'], ['c']], "[['a', 'b'], ['c']]"),
+        )
+        for value, display in tests:
+            with self.subTest(value=value, display=display):
+                instance = MyModel(field=value)
+                self.assertEqual(instance.get_field_display(), display)
+
+
 class TestSaveLoad(PostgreSQLTestCase):
 
     def test_integer(self):
@@ -140,17 +191,23 @@ class TestSaveLoad(PostgreSQLTestCase):
         self.assertEqual(field.model, IntegerArrayModel)
         self.assertEqual(field.base_field.model, IntegerArrayModel)
 
+    def test_nested_nullable_base_field(self):
+        instance = NullableIntegerArrayModel.objects.create(
+            field_nested=[[None, None], [None, None]],
+        )
+        self.assertEqual(instance.field_nested, [[None, None], [None, None]])
+
 
 class TestQuerying(PostgreSQLTestCase):
 
     @classmethod
     def setUpTestData(cls):
         cls.objs = NullableIntegerArrayModel.objects.bulk_create([
-            NullableIntegerArrayModel(field=[1]),
-            NullableIntegerArrayModel(field=[2]),
-            NullableIntegerArrayModel(field=[2, 3]),
-            NullableIntegerArrayModel(field=[20, 30, 40]),
-            NullableIntegerArrayModel(field=None),
+            NullableIntegerArrayModel(order=1, field=[1]),
+            NullableIntegerArrayModel(order=2, field=[2]),
+            NullableIntegerArrayModel(order=3, field=[2, 3]),
+            NullableIntegerArrayModel(order=4, field=[20, 30, 40]),
+            NullableIntegerArrayModel(order=5, field=None),
         ])
 
     def test_empty_list(self):
@@ -167,6 +224,12 @@ class TestQuerying(PostgreSQLTestCase):
             self.objs[:1]
         )
 
+    def test_exact_with_expression(self):
+        self.assertSequenceEqual(
+            NullableIntegerArrayModel.objects.filter(field__exact=[Value(1)]),
+            self.objs[:1],
+        )
+
     def test_exact_charfield(self):
         instance = CharArrayModel.objects.create(field=['text'])
         self.assertSequenceEqual(
@@ -237,15 +300,10 @@ class TestQuerying(PostgreSQLTestCase):
             self.objs[:2]
         )
 
-    @unittest.expectedFailure
     def test_contained_by_including_F_object(self):
-        # This test asserts that Array objects passed to filters can be
-        # constructed to contain F objects. This currently doesn't work as the
-        # psycopg2 mogrify method that generates the ARRAY() syntax is
-        # expecting literals, not column references (#27095).
         self.assertSequenceEqual(
-            NullableIntegerArrayModel.objects.filter(field__contained_by=[models.F('id'), 2]),
-            self.objs[:2]
+            NullableIntegerArrayModel.objects.filter(field__contained_by=[models.F('order'), 2]),
+            self.objs[:3],
         )
 
     def test_contains(self):
@@ -254,6 +312,27 @@ class TestQuerying(PostgreSQLTestCase):
             self.objs[1:3]
         )
 
+    def test_contains_subquery(self):
+        IntegerArrayModel.objects.create(field=[2, 3])
+        inner_qs = IntegerArrayModel.objects.values_list('field', flat=True)
+        self.assertSequenceEqual(
+            NullableIntegerArrayModel.objects.filter(field__contains=inner_qs[:1]),
+            self.objs[2:3],
+        )
+        inner_qs = IntegerArrayModel.objects.filter(field__contains=OuterRef('field'))
+        self.assertSequenceEqual(
+            NullableIntegerArrayModel.objects.filter(Exists(inner_qs)),
+            self.objs[1:3],
+        )
+
+    def test_contains_including_expression(self):
+        self.assertSequenceEqual(
+            NullableIntegerArrayModel.objects.filter(
+                field__contains=[2, Value(6) / Value(2)],
+            ),
+            self.objs[2:3],
+        )
+
     def test_icontains(self):
         # Using the __icontains lookup with ArrayField is inefficient.
         instance = CharArrayModel.objects.create(field=['FoO'])
@@ -281,6 +360,18 @@ class TestQuerying(PostgreSQLTestCase):
             []
         )
 
+    def test_overlap_charfield_including_expression(self):
+        obj_1 = CharArrayModel.objects.create(field=['TEXT', 'lower text'])
+        obj_2 = CharArrayModel.objects.create(field=['lower text', 'TEXT'])
+        CharArrayModel.objects.create(field=['lower text', 'text'])
+        self.assertSequenceEqual(
+            CharArrayModel.objects.filter(field__overlap=[
+                Upper(Value('text')),
+                'other',
+            ]),
+            [obj_1, obj_2],
+        )
+
     def test_lookups_autofield_array(self):
         qs = NullableIntegerArrayModel.objects.filter(
             field__0__isnull=False,
@@ -341,6 +432,13 @@ class TestQuerying(PostgreSQLTestCase):
             self.objs[:1],
         )
 
+    def test_index_annotation(self):
+        qs = NullableIntegerArrayModel.objects.annotate(second=models.F('field__1'))
+        self.assertCountEqual(
+            qs.values_list('second', flat=True),
+            [None, None, None, 3, 30],
+        )
+
     def test_overlap(self):
         self.assertSequenceEqual(
             NullableIntegerArrayModel.objects.filter(field__overlap=[1, 2]),
@@ -402,6 +500,15 @@ class TestQuerying(PostgreSQLTestCase):
             self.objs[2:3],
         )
 
+    def test_slice_annotation(self):
+        qs = NullableIntegerArrayModel.objects.annotate(
+            first_two=models.F('field__0_2'),
+        )
+        self.assertCountEqual(
+            qs.values_list('first_two', flat=True),
+            [None, [1], [2], [2, 3], [20, 30]],
+        )
+
     def test_usage_in_subquery(self):
         self.assertSequenceEqual(
             NullableIntegerArrayModel.objects.filter(
@@ -433,13 +540,75 @@ class TestQuerying(PostgreSQLTestCase):
         value = models.Value([1], output_field=ArrayField(models.IntegerField()))
         self.assertEqual(
             NullableIntegerArrayModel.objects.annotate(
-                array_length=models.Func(value, 1, function='ARRAY_LENGTH'),
+                array_length=models.Func(
+                    value, 1, function='ARRAY_LENGTH', output_field=models.IntegerField(),
+                ),
             ).values('array_length').annotate(
                 count=models.Count('pk'),
             ).get()['array_length'],
             1,
         )
 
+    def test_filter_by_array_subquery(self):
+        inner_qs = NullableIntegerArrayModel.objects.filter(
+            field__len=models.OuterRef('field__len'),
+        ).values('field')
+        self.assertSequenceEqual(
+            NullableIntegerArrayModel.objects.alias(
+                same_sized_fields=ArraySubquery(inner_qs),
+            ).filter(same_sized_fields__len__gt=1),
+            self.objs[0:2],
+        )
+
+    def test_annotated_array_subquery(self):
+        inner_qs = NullableIntegerArrayModel.objects.exclude(
+            pk=models.OuterRef('pk')
+        ).values('order')
+        self.assertSequenceEqual(
+            NullableIntegerArrayModel.objects.annotate(
+                sibling_ids=ArraySubquery(inner_qs),
+            ).get(order=1).sibling_ids,
+            [2, 3, 4, 5],
+        )
+
+    def test_group_by_with_annotated_array_subquery(self):
+        inner_qs = NullableIntegerArrayModel.objects.exclude(
+            pk=models.OuterRef('pk')
+        ).values('order')
+        self.assertSequenceEqual(
+            NullableIntegerArrayModel.objects.annotate(
+                sibling_ids=ArraySubquery(inner_qs),
+                sibling_count=models.Max('sibling_ids__len'),
+            ).values_list('sibling_count', flat=True),
+            [len(self.objs) - 1] * len(self.objs),
+        )
+
+    def test_annotated_ordered_array_subquery(self):
+        inner_qs = NullableIntegerArrayModel.objects.order_by('-order').values('order')
+        self.assertSequenceEqual(
+            NullableIntegerArrayModel.objects.annotate(
+                ids=ArraySubquery(inner_qs),
+            ).first().ids,
+            [5, 4, 3, 2, 1],
+        )
+
+    def test_annotated_array_subquery_with_json_objects(self):
+        inner_qs = NullableIntegerArrayModel.objects.exclude(
+            pk=models.OuterRef('pk')
+        ).values(json=JSONObject(order='order', field='field'))
+        siblings_json = NullableIntegerArrayModel.objects.annotate(
+            siblings_json=ArraySubquery(inner_qs),
+        ).values_list('siblings_json', flat=True).get(order=1)
+        self.assertSequenceEqual(
+            siblings_json,
+            [
+                {'field': [2], 'order': 2},
+                {'field': [2, 3], 'order': 3},
+                {'field': [20, 30, 40], 'order': 4},
+                {'field': None, 'order': 5},
+            ],
+        )
+
 
 class TestDateTimeExactQuerying(PostgreSQLTestCase):
 
@@ -551,7 +720,7 @@ class TestChecks(PostgreSQLSimpleTestCase):
                 ),
                 hint='Use a callable instead, e.g., use `list` instead of `[]`.',
                 obj=MyModel._meta.get_field('field'),
-                id='postgres.E003',
+                id='fields.E010',
             )
         ])
 
@@ -583,6 +752,20 @@ class TestChecks(PostgreSQLSimpleTestCase):
         self.assertEqual(errors[0].id, 'postgres.E001')
         self.assertIn('max_length', errors[0].msg)
 
+    def test_choices_tuple_list(self):
+        class MyModel(PostgreSQLModel):
+            field = ArrayField(
+                models.CharField(max_length=16),
+                choices=[
+                    [
+                        'Media',
+                        [(['vinyl', 'cd'], 'Audio'), (('vhs', 'dvd'), 'Video')],
+                    ],
+                    (['mp3', 'mp4'], 'Digital'),
+                ],
+            )
+        self.assertEqual(MyModel._meta.get_field('field').check(), [])
+
 
 @unittest.skipUnless(connection.vendor == 'postgresql', "PostgreSQL specific tests")
 class TestMigrations(TransactionTestCase):
@@ -961,9 +1144,39 @@ class TestSplitFormField(PostgreSQLSimpleTestCase):
                 model = IntegerArrayModel
                 fields = ('field',)
 
-        obj = IntegerArrayModel(field=[1, 2])
-        form = Form({'field_0': '1', 'field_1': '2'}, instance=obj)
-        self.assertFalse(form.has_changed())
+        tests = [
+            ({}, {'field_0': '', 'field_1': ''}, True),
+            ({'field': None}, {'field_0': '', 'field_1': ''}, True),
+            ({'field': [1]}, {'field_0': '', 'field_1': ''}, True),
+            ({'field': [1]}, {'field_0': '1', 'field_1': '0'}, True),
+            ({'field': [1, 2]}, {'field_0': '1', 'field_1': '2'}, False),
+            ({'field': [1, 2]}, {'field_0': 'a', 'field_1': 'b'}, True),
+        ]
+        for initial, data, expected_result in tests:
+            with self.subTest(initial=initial, data=data):
+                obj = IntegerArrayModel(**initial)
+                form = Form(data, instance=obj)
+                self.assertIs(form.has_changed(), expected_result)
+
+    def test_splitarrayfield_remove_trailing_nulls_has_changed(self):
+        class Form(forms.ModelForm):
+            field = SplitArrayField(forms.IntegerField(), required=False, size=2, remove_trailing_nulls=True)
+
+            class Meta:
+                model = IntegerArrayModel
+                fields = ('field',)
+
+        tests = [
+            ({}, {'field_0': '', 'field_1': ''}, False),
+            ({'field': None}, {'field_0': '', 'field_1': ''}, False),
+            ({'field': []}, {'field_0': '', 'field_1': ''}, False),
+            ({'field': [1]}, {'field_0': '1', 'field_1': ''}, False),
+        ]
+        for initial, data, expected_result in tests:
+            with self.subTest(initial=initial, data=data):
+                obj = IntegerArrayModel(**initial)
+                form = Form(data, instance=obj)
+                self.assertIs(form.has_changed(), expected_result)
 
 
 class TestSplitFormWidget(PostgreSQLWidgetTestCase):
