@@ -1,14 +1,13 @@
 import os
 from datetime import date
-from unittest import skipUnless
 
-from django.conf import settings
 from django.contrib.sitemaps import Sitemap
 from django.contrib.sites.models import Site
 from django.core.exceptions import ImproperlyConfigured
-from django.test import modify_settings, override_settings
+from django.test import ignore_warnings, modify_settings, override_settings
+from django.utils import translation
+from django.utils.deprecation import RemovedInDjango50Warning
 from django.utils.formats import localize
-from django.utils.translation import activate, deactivate
 
 from .base import SitemapTestsBase
 from .models import TestModel
@@ -69,7 +68,7 @@ class HTTPSitemapTests(SitemapTestsBase):
         "A simple sitemap section can be rendered"
         response = self.client.get('/simple/sitemap-simple.xml')
         expected_content = """<?xml version="1.0" encoding="UTF-8"?>
-<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">
+<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" xmlns:xhtml="http://www.w3.org/1999/xhtml">
 <url><loc>%s/location/</loc><lastmod>%s</lastmod><changefreq>never</changefreq><priority>0.5</priority></url>
 </urlset>
 """ % (self.base_url, date.today())
@@ -94,7 +93,7 @@ class HTTPSitemapTests(SitemapTestsBase):
         "A simple sitemap can be rendered"
         response = self.client.get('/simple/sitemap.xml')
         expected_content = """<?xml version="1.0" encoding="UTF-8"?>
-<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">
+<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" xmlns:xhtml="http://www.w3.org/1999/xhtml">
 <url><loc>%s/location/</loc><lastmod>%s</lastmod><changefreq>never</changefreq><priority>0.5</priority></url>
 </urlset>
 """ % (self.base_url, date.today())
@@ -118,14 +117,14 @@ class HTTPSitemapTests(SitemapTestsBase):
     def test_sitemap_last_modified(self):
         "Last-Modified header is set correctly"
         response = self.client.get('/lastmod/sitemap.xml')
-        self.assertEqual(response['Last-Modified'], 'Wed, 13 Mar 2013 10:00:00 GMT')
+        self.assertEqual(response.headers['Last-Modified'], 'Wed, 13 Mar 2013 10:00:00 GMT')
 
     def test_sitemap_last_modified_date(self):
         """
         The Last-Modified header should be support dates (without time).
         """
         response = self.client.get('/lastmod/date-sitemap.xml')
-        self.assertEqual(response['Last-Modified'], 'Wed, 13 Mar 2013 00:00:00 GMT')
+        self.assertEqual(response.headers['Last-Modified'], 'Wed, 13 Mar 2013 00:00:00 GMT')
 
     def test_sitemap_last_modified_tz(self):
         """
@@ -133,7 +132,7 @@ class HTTPSitemapTests(SitemapTestsBase):
         to GMT.
         """
         response = self.client.get('/lastmod/tz-sitemap.xml')
-        self.assertEqual(response['Last-Modified'], 'Wed, 13 Mar 2013 15:00:00 GMT')
+        self.assertEqual(response.headers['Last-Modified'], 'Wed, 13 Mar 2013 15:00:00 GMT')
 
     def test_sitemap_last_modified_missing(self):
         "Last-Modified header is missing when sitemap has no lastmod"
@@ -167,7 +166,7 @@ class HTTPSitemapTests(SitemapTestsBase):
         Test sitemaps are sorted by lastmod in ascending order.
         """
         response = self.client.get('/lastmod-sitemaps/ascending.xml')
-        self.assertEqual(response['Last-Modified'], 'Sat, 20 Apr 2013 05:00:00 GMT')
+        self.assertEqual(response.headers['Last-Modified'], 'Sat, 20 Apr 2013 05:00:00 GMT')
 
     def test_sitemaps_lastmod_descending(self):
         """
@@ -175,20 +174,17 @@ class HTTPSitemapTests(SitemapTestsBase):
         Test sitemaps are sorted by lastmod in descending order.
         """
         response = self.client.get('/lastmod-sitemaps/descending.xml')
-        self.assertEqual(response['Last-Modified'], 'Sat, 20 Apr 2013 05:00:00 GMT')
+        self.assertEqual(response.headers['Last-Modified'], 'Sat, 20 Apr 2013 05:00:00 GMT')
 
-    @skipUnless(settings.USE_I18N, "Internationalization is not enabled")
-    @override_settings(USE_L10N=True)
+    @override_settings(USE_I18N=True)
     def test_localized_priority(self):
-        "The priority value should not be localized (Refs #14164)"
-        activate('fr')
-        self.assertEqual('0,3', localize(0.3))
-
-        # Priorities haven't been rendered in localized format.
-        response = self.client.get('/simple/sitemap.xml')
-        self.assertContains(response, '<priority>0.5</priority>')
-        self.assertContains(response, '<lastmod>%s</lastmod>' % date.today())
-        deactivate()
+        """The priority value should not be localized."""
+        with translation.override('fr'):
+            self.assertEqual('0,3', localize(0.3))
+            # Priorities aren't rendered in localized format.
+            response = self.client.get('/simple/sitemap.xml')
+            self.assertContains(response, '<priority>0.5</priority>')
+            self.assertContains(response, '<lastmod>%s</lastmod>' % date.today())
 
     @modify_settings(INSTALLED_APPS={'remove': 'django.contrib.sites'})
     def test_requestsite_sitemap(self):
@@ -196,12 +192,13 @@ class HTTPSitemapTests(SitemapTestsBase):
         # doesn't raise an exception.
         response = self.client.get('/simple/sitemap.xml')
         expected_content = """<?xml version="1.0" encoding="UTF-8"?>
-<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">
+<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" xmlns:xhtml="http://www.w3.org/1999/xhtml">
 <url><loc>http://testserver/location/</loc><lastmod>%s</lastmod><changefreq>never</changefreq><priority>0.5</priority></url>
 </urlset>
 """ % date.today()
         self.assertXMLEqual(response.content.decode(), expected_content)
 
+    @ignore_warnings(category=RemovedInDjango50Warning)
     def test_sitemap_get_urls_no_site_1(self):
         """
         Check we get ImproperlyConfigured if we don't pass a site object to
@@ -212,6 +209,7 @@ class HTTPSitemapTests(SitemapTestsBase):
             Sitemap().get_urls()
 
     @modify_settings(INSTALLED_APPS={'remove': 'django.contrib.sites'})
+    @ignore_warnings(category=RemovedInDjango50Warning)
     def test_sitemap_get_urls_no_site_2(self):
         """
         Check we get ImproperlyConfigured when we don't pass a site object to
@@ -221,6 +219,7 @@ class HTTPSitemapTests(SitemapTestsBase):
         with self.assertRaisesMessage(ImproperlyConfigured, self.use_sitemap_err_msg):
             Sitemap().get_urls()
 
+    @ignore_warnings(category=RemovedInDjango50Warning)
     def test_sitemap_item(self):
         """
         Check to make sure that the raw item is included with each
@@ -248,30 +247,108 @@ class HTTPSitemapTests(SitemapTestsBase):
 
     def test_x_robots_sitemap(self):
         response = self.client.get('/simple/index.xml')
-        self.assertEqual(response['X-Robots-Tag'], 'noindex, noodp, noarchive')
+        self.assertEqual(response.headers['X-Robots-Tag'], 'noindex, noodp, noarchive')
 
         response = self.client.get('/simple/sitemap.xml')
-        self.assertEqual(response['X-Robots-Tag'], 'noindex, noodp, noarchive')
+        self.assertEqual(response.headers['X-Robots-Tag'], 'noindex, noodp, noarchive')
 
     def test_empty_sitemap(self):
         response = self.client.get('/empty/sitemap.xml')
         self.assertEqual(response.status_code, 200)
 
     @override_settings(LANGUAGES=(('en', 'English'), ('pt', 'Portuguese')))
-    def test_simple_i18nsitemap_index(self):
-        "A simple i18n sitemap index can be rendered"
-        response = self.client.get('/simple/i18n.xml')
+    def test_simple_i18n_sitemap_index(self):
+        """
+        A simple i18n sitemap index can be rendered, without logging variable
+        lookup errors.
+        """
+        with self.assertNoLogs('django.template', 'DEBUG'):
+            response = self.client.get('/simple/i18n.xml')
         expected_content = """<?xml version="1.0" encoding="UTF-8"?>
-<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">
+<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" xmlns:xhtml="http://www.w3.org/1999/xhtml">
 <url><loc>{0}/en/i18n/testmodel/{1}/</loc><changefreq>never</changefreq><priority>0.5</priority></url><url><loc>{0}/pt/i18n/testmodel/{1}/</loc><changefreq>never</changefreq><priority>0.5</priority></url>
 </urlset>
 """.format(self.base_url, self.i18n_model.pk)
         self.assertXMLEqual(response.content.decode(), expected_content)
 
+    @override_settings(LANGUAGES=(('en', 'English'), ('pt', 'Portuguese')))
+    def test_alternate_i18n_sitemap_index(self):
+        """
+        A i18n sitemap with alternate/hreflang links can be rendered.
+        """
+        response = self.client.get('/alternates/i18n.xml')
+        url, pk = self.base_url, self.i18n_model.pk
+        expected_urls = f"""
+<url><loc>{url}/en/i18n/testmodel/{pk}/</loc><changefreq>never</changefreq><priority>0.5</priority>
+<xhtml:link rel="alternate" hreflang="en" href="{url}/en/i18n/testmodel/{pk}/"/>
+<xhtml:link rel="alternate" hreflang="pt" href="{url}/pt/i18n/testmodel/{pk}/"/>
+</url>
+<url><loc>{url}/pt/i18n/testmodel/{pk}/</loc><changefreq>never</changefreq><priority>0.5</priority>
+<xhtml:link rel="alternate" hreflang="en" href="{url}/en/i18n/testmodel/{pk}/"/>
+<xhtml:link rel="alternate" hreflang="pt" href="{url}/pt/i18n/testmodel/{pk}/"/>
+</url>
+""".replace('\n', '')
+        expected_content = f"""<?xml version="1.0" encoding="UTF-8"?>
+<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" xmlns:xhtml="http://www.w3.org/1999/xhtml">
+{expected_urls}
+</urlset>
+"""
+        self.assertXMLEqual(response.content.decode(), expected_content)
+
+    @override_settings(LANGUAGES=(('en', 'English'), ('pt', 'Portuguese'), ('es', 'Spanish')))
+    def test_alternate_i18n_sitemap_limited(self):
+        """
+        A i18n sitemap index with limited languages can be rendered.
+        """
+        response = self.client.get('/limited/i18n.xml')
+        url, pk = self.base_url, self.i18n_model.pk
+        expected_urls = f"""
+<url><loc>{url}/en/i18n/testmodel/{pk}/</loc><changefreq>never</changefreq><priority>0.5</priority>
+<xhtml:link rel="alternate" hreflang="en" href="{url}/en/i18n/testmodel/{pk}/"/>
+<xhtml:link rel="alternate" hreflang="es" href="{url}/es/i18n/testmodel/{pk}/"/>
+</url>
+<url><loc>{url}/es/i18n/testmodel/{pk}/</loc><changefreq>never</changefreq><priority>0.5</priority>
+<xhtml:link rel="alternate" hreflang="en" href="{url}/en/i18n/testmodel/{pk}/"/>
+<xhtml:link rel="alternate" hreflang="es" href="{url}/es/i18n/testmodel/{pk}/"/>
+</url>
+""".replace('\n', '')
+        expected_content = f"""<?xml version="1.0" encoding="UTF-8"?>
+<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" xmlns:xhtml="http://www.w3.org/1999/xhtml">
+{expected_urls}
+</urlset>
+"""
+        self.assertXMLEqual(response.content.decode(), expected_content)
+
+    @override_settings(LANGUAGES=(('en', 'English'), ('pt', 'Portuguese')))
+    def test_alternate_i18n_sitemap_xdefault(self):
+        """
+        A i18n sitemap index with x-default can be rendered.
+        """
+        response = self.client.get('/x-default/i18n.xml')
+        url, pk = self.base_url, self.i18n_model.pk
+        expected_urls = f"""
+<url><loc>{url}/en/i18n/testmodel/{pk}/</loc><changefreq>never</changefreq><priority>0.5</priority>
+<xhtml:link rel="alternate" hreflang="en" href="{url}/en/i18n/testmodel/{pk}/"/>
+<xhtml:link rel="alternate" hreflang="pt" href="{url}/pt/i18n/testmodel/{pk}/"/>
+<xhtml:link rel="alternate" hreflang="x-default" href="{url}/i18n/testmodel/{pk}/"/>
+</url>
+<url><loc>{url}/pt/i18n/testmodel/{pk}/</loc><changefreq>never</changefreq><priority>0.5</priority>
+<xhtml:link rel="alternate" hreflang="en" href="{url}/en/i18n/testmodel/{pk}/"/>
+<xhtml:link rel="alternate" hreflang="pt" href="{url}/pt/i18n/testmodel/{pk}/"/>
+<xhtml:link rel="alternate" hreflang="x-default" href="{url}/i18n/testmodel/{pk}/"/>
+</url>
+""".replace('\n', '')
+        expected_content = f"""<?xml version="1.0" encoding="UTF-8"?>
+<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" xmlns:xhtml="http://www.w3.org/1999/xhtml">
+{expected_urls}
+</urlset>
+"""
+        self.assertXMLEqual(response.content.decode(), expected_content)
+
     def test_sitemap_without_entries(self):
         response = self.client.get('/sitemap-without-entries/sitemap.xml')
         expected_content = """<?xml version="1.0" encoding="UTF-8"?>
-<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">
+<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" xmlns:xhtml="http://www.w3.org/1999/xhtml">
 
 </urlset>"""
         self.assertXMLEqual(response.content.decode(), expected_content)
