@@ -1,20 +1,17 @@
-from __future__ import unicode_literals
-
 import os.path
+from pathlib import Path
 
-from django.utils._os import upath
-
-TEST_ROOT = os.path.dirname(upath(__file__))
+TEST_ROOT = os.path.dirname(__file__)
 
 TEST_SETTINGS = {
-    'DEBUG': True,
-    'MEDIA_URL': '/media/',
-    'STATIC_URL': '/static/',
+    'MEDIA_URL': 'media/',
+    'STATIC_URL': 'static/',
     'MEDIA_ROOT': os.path.join(TEST_ROOT, 'project', 'site_media', 'media'),
     'STATIC_ROOT': os.path.join(TEST_ROOT, 'project', 'site_media', 'static'),
     'STATICFILES_DIRS': [
         os.path.join(TEST_ROOT, 'project', 'documents'),
         ('prefix', os.path.join(TEST_ROOT, 'project', 'prefixed')),
+        Path(TEST_ROOT) / 'project' / 'pathlib',
     ],
     'STATICFILES_FINDERS': [
         'django.contrib.staticfiles.finders.FileSystemFinder',
@@ -27,4 +24,7 @@ TEST_SETTINGS = {
         'staticfiles_tests.apps.test',
         'staticfiles_tests.apps.no_label',
     ],
+    # In particular, AuthenticationMiddleware can't be used because
+    # contrib.auth isn't in INSTALLED_APPS.
+    'MIDDLEWARE': [],
 }
