@@ -4,6 +4,7 @@ import shutil
 import tempfile
 import unittest
 from io import StringIO
+from pathlib import Path
 from unittest import mock
 
 from admin_scripts.tests import AdminScriptTestCase
@@ -15,6 +16,7 @@ from django.contrib.staticfiles.management.commands import (
 )
 from django.core.exceptions import ImproperlyConfigured
 from django.core.management import CommandError, call_command
+from django.core.management.base import SystemCheckError
 from django.test import RequestFactory, override_settings
 from django.test.utils import extend_sys_path
 from django.utils import timezone
@@ -102,6 +104,7 @@ class TestFindStatic(TestDefaults, CollectionTestCase):
         # FileSystemFinder searched locations
         self.assertIn(TEST_SETTINGS['STATICFILES_DIRS'][1][1], searched_locations)
         self.assertIn(TEST_SETTINGS['STATICFILES_DIRS'][0], searched_locations)
+        self.assertIn(str(TEST_SETTINGS['STATICFILES_DIRS'][2]), searched_locations)
         # DefaultStorageFinder searched locations
         self.assertIn(
             os.path.join('staticfiles_tests', 'project', 'site_media', 'media'),
@@ -143,6 +146,12 @@ class TestConfiguration(StaticFilesTestCase):
             collectstatic.staticfiles_storage = staticfiles_storage
             storage.staticfiles_storage = staticfiles_storage
 
+    @override_settings(STATICFILES_DIRS=('test'))
+    def test_collectstatis_check(self):
+        msg = 'The STATICFILES_DIRS setting is not a tuple or list.'
+        with self.assertRaisesMessage(SystemCheckError, msg):
+            call_command('collectstatic', skip_checks=False)
+
 
 class TestCollectionHelpSubcommand(AdminScriptTestCase):
     @override_settings(STATIC_ROOT=None)
@@ -174,6 +183,15 @@ class TestCollection(TestDefaults, CollectionTestCase):
         self.assertFileNotFound('test/backup~')
         self.assertFileNotFound('test/CVS')
 
+    def test_pathlib(self):
+        self.assertFileContains('pathlib.txt', 'pathlib')
+
+
+class TestCollectionPathLib(TestCollection):
+    def mkdtemp(self):
+        tmp_dir = super().mkdtemp()
+        return Path(tmp_dir)
+
 
 class TestCollectionVerbosity(CollectionTestCase):
     copying_msg = 'Copying '
@@ -293,10 +311,10 @@ class TestInteractiveMessages(CollectionTestCase):
                 call_command('collectstatic', interactive=True)
 
 
-class TestCollectionExcludeNoDefaultIgnore(TestDefaults, CollectionTestCase):
+class TestCollectionNoDefaultIgnore(TestDefaults, CollectionTestCase):
     """
-    Test ``--exclude-dirs`` and ``--no-default-ignore`` options of the
-    ``collectstatic`` management command.
+    The ``--no-default-ignore`` option of the ``collectstatic`` management
+    command.
     """
     def run_collectstatic(self):
         super().run_collectstatic(use_default_ignore_patterns=False)
