@@ -1,3 +1,4 @@
+import logging
 import os
 import unittest
 import warnings
@@ -12,7 +13,10 @@ from django.core.files.storage import default_storage
 from django.db import (
     IntegrityError, connection, connections, models, router, transaction,
 )
-from django.forms import EmailField, IntegerField
+from django.forms import (
+    CharField, EmailField, Form, IntegerField, ValidationError,
+    formset_factory,
+)
 from django.http import HttpResponse
 from django.template.loader import render_to_string
 from django.test import (
@@ -26,6 +30,7 @@ from django.test.utils import (
 )
 from django.urls import NoReverseMatch, path, reverse, reverse_lazy
 from django.utils.deprecation import RemovedInDjango41Warning
+from django.utils.log import DEFAULT_LOGGING
 
 from .models import Car, Person, PossessedCar
 from .views import empty_response
@@ -289,7 +294,10 @@ class AssertQuerysetEqualTests(TestCase):
     def test_undefined_order(self):
         # Using an unordered queryset with more than one ordered value
         # is an error.
-        msg = 'Trying to compare non-ordered queryset against more than one ordered values'
+        msg = (
+            'Trying to compare non-ordered queryset against more than one '
+            'ordered value.'
+        )
         with self.assertRaisesMessage(ValueError, msg):
             self.assertQuerysetEqual(
                 Person.objects.all(),
@@ -325,6 +333,37 @@ class AssertQuerysetEqualTests(TestCase):
             ordered=False
         )
 
+    def test_maxdiff(self):
+        names = ['Joe Smith %s' % i for i in range(20)]
+        Person.objects.bulk_create([Person(name=name) for name in names])
+        names.append('Extra Person')
+
+        with self.assertRaises(AssertionError) as ctx:
+            self.assertQuerysetEqual(
+                Person.objects.filter(name__startswith='Joe'),
+                names,
+                ordered=False,
+                transform=lambda p: p.name,
+            )
+        self.assertIn('Set self.maxDiff to None to see it.', str(ctx.exception))
+
+        original = self.maxDiff
+        self.maxDiff = None
+        try:
+            with self.assertRaises(AssertionError) as ctx:
+                self.assertQuerysetEqual(
+                    Person.objects.filter(name__startswith='Joe'),
+                    names,
+                    ordered=False,
+                    transform=lambda p: p.name,
+                )
+        finally:
+            self.maxDiff = original
+        exception_msg = str(ctx.exception)
+        self.assertNotIn('Set self.maxDiff to None to see it.', exception_msg)
+        for name in names:
+            self.assertIn(name, exception_msg)
+
 
 class AssertQuerysetEqualDeprecationTests(TestCase):
     @classmethod
@@ -678,6 +717,27 @@ class HTMLEqualTests(SimpleTestCase):
             with self.subTest(html1):
                 self.assertHTMLEqual(html1, html2)
 
+    def test_boolean_attribute(self):
+        html1 = '<input checked>'
+        html2 = '<input checked="">'
+        html3 = '<input checked="checked">'
+        self.assertHTMLEqual(html1, html2)
+        self.assertHTMLEqual(html1, html3)
+        self.assertHTMLEqual(html2, html3)
+        self.assertHTMLNotEqual(html1, '<input checked="invalid">')
+        self.assertEqual(str(parse_html(html1)), '<input checked>')
+        self.assertEqual(str(parse_html(html2)), '<input checked>')
+        self.assertEqual(str(parse_html(html3)), '<input checked>')
+
+    def test_non_boolean_attibutes(self):
+        html1 = '<input value>'
+        html2 = '<input value="">'
+        html3 = '<input value="value">'
+        self.assertHTMLEqual(html1, html2)
+        self.assertHTMLNotEqual(html1, html3)
+        self.assertEqual(str(parse_html(html1)), '<input value="">')
+        self.assertEqual(str(parse_html(html2)), '<input value="">')
+
     def test_normalize_refs(self):
         pairs = [
             ('&#39;', '&#x27;'),
@@ -1071,6 +1131,47 @@ class AssertWarnsMessageTests(SimpleTestCase):
             func1()
 
 
+# TODO: Remove when dropping support for PY39.
+class AssertNoLogsTest(SimpleTestCase):
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+        logging.config.dictConfig(DEFAULT_LOGGING)
+        cls.addClassCleanup(logging.config.dictConfig, settings.LOGGING)
+
+    def setUp(self):
+        self.logger = logging.getLogger('django')
+
+    @override_settings(DEBUG=True)
+    def test_fails_when_log_emitted(self):
+        msg = "Unexpected logs found: ['INFO:django:FAIL!']"
+        with self.assertRaisesMessage(AssertionError, msg):
+            with self.assertNoLogs('django', 'INFO'):
+                self.logger.info('FAIL!')
+
+    @override_settings(DEBUG=True)
+    def test_text_level(self):
+        with self.assertNoLogs('django', 'INFO'):
+            self.logger.debug('DEBUG logs are ignored.')
+
+    @override_settings(DEBUG=True)
+    def test_int_level(self):
+        with self.assertNoLogs('django', logging.INFO):
+            self.logger.debug('DEBUG logs are ignored.')
+
+    @override_settings(DEBUG=True)
+    def test_default_level(self):
+        with self.assertNoLogs('django'):
+            self.logger.debug('DEBUG logs are ignored.')
+
+    @override_settings(DEBUG=True)
+    def test_does_not_hide_other_failures(self):
+        msg = '1 != 2'
+        with self.assertRaisesMessage(AssertionError, msg):
+            with self.assertNoLogs('django'):
+                self.assertEqual(1, 2)
+
+
 class AssertFieldOutputTests(SimpleTestCase):
 
     def test_assert_field_output(self):
@@ -1143,6 +1244,44 @@ class AssertURLEqualTests(SimpleTestCase):
             )
 
 
+class TestForm(Form):
+    field = CharField()
+
+    def clean_field(self):
+        value = self.cleaned_data.get('field', '')
+        if value == 'invalid':
+            raise ValidationError('invalid value')
+        return value
+
+
+class TestFormset(formset_factory(TestForm)):
+    @classmethod
+    def _get_cleaned_formset(cls, field_value):
+        formset = cls({
+            'form-TOTAL_FORMS': '1',
+            'form-INITIAL_FORMS': '0',
+            'form-0-field': field_value,
+        })
+        formset.full_clean()
+        return formset
+
+    @classmethod
+    def invalid(cls):
+        return cls._get_cleaned_formset('invalid')
+
+
+class AssertFormsetErrorTests(SimpleTestCase):
+    def test_formset_named_form(self):
+        formset = TestFormset.invalid()
+        # The mocked context emulates the template-based rendering of the
+        # formset.
+        response = mock.Mock(context=[
+            {'form': formset},
+            {'form': formset.management_form},
+        ])
+        self.assertFormsetError(response, 'form', 0, 'field', 'invalid value')
+
+
 class FirstUrls:
     urlpatterns = [path('first/', empty_response, name='first')]
 
@@ -1404,6 +1543,13 @@ class CaptureOnCommitCallbacksTests(TestCase):
 
         self.assertEqual(callbacks, [])
 
+    def test_execute_recursive(self):
+        with self.captureOnCommitCallbacks(execute=True) as callbacks:
+            transaction.on_commit(self.enqueue_callback)
+
+        self.assertEqual(len(callbacks), 2)
+        self.assertIs(self.callback_called, True)
+
 
 class DisallowedDatabaseQueriesTests(SimpleTestCase):
     def test_disallowed_database_connections(self):
