@@ -1,7 +1,7 @@
 from unittest import mock
 
 from django.test import SimpleTestCase
-from django.utils.functional import cached_property, lazy
+from django.utils.functional import cached_property, classproperty, lazy
 
 
 class FunctionalTests(SimpleTestCase):
@@ -184,6 +184,11 @@ class FunctionalTests(SimpleTestCase):
         with self.assertRaisesMessage(TypeError, msg):
             Foo().cp
 
+    def test_lazy_add(self):
+        lazy_4 = lazy(lambda: 4, int)
+        lazy_5 = lazy(lambda: 5, int)
+        self.assertEqual(lazy_4() + lazy_5(), 9)
+
     def test_lazy_equality(self):
         """
         == and != work correctly for Promises.
@@ -218,3 +223,45 @@ class FunctionalTests(SimpleTestCase):
         with mock.patch.object(__proxy__, '__prepare_class__') as mocked:
             lazified()
             mocked.assert_not_called()
+
+    def test_lazy_bytes_and_str_result_classes(self):
+        lazy_obj = lazy(lambda: 'test', str, bytes)
+        msg = 'Cannot call lazy() with both bytes and text return types.'
+        with self.assertRaisesMessage(ValueError, msg):
+            lazy_obj()
+
+    def test_classproperty_getter(self):
+        class Foo:
+            foo_attr = 123
+
+            def __init__(self):
+                self.foo_attr = 456
+
+            @classproperty
+            def foo(cls):
+                return cls.foo_attr
+
+        class Bar:
+            bar = classproperty()
+
+            @bar.getter
+            def bar(cls):
+                return 123
+
+        self.assertEqual(Foo.foo, 123)
+        self.assertEqual(Foo().foo, 123)
+        self.assertEqual(Bar.bar, 123)
+        self.assertEqual(Bar().bar, 123)
+
+    def test_classproperty_override_getter(self):
+        class Foo:
+            @classproperty
+            def foo(cls):
+                return 123
+
+            @foo.getter
+            def foo(cls):
+                return 456
+
+        self.assertEqual(Foo.foo, 456)
+        self.assertEqual(Foo().foo, 456)
